/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.controller;

import java.beans.PropertyEditor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.springlayout.util.DateFormatUtil;
import net.sf.springlayout.web.bind.LayoutServletRequestDataBinder;
import net.sf.springlayout.web.controller.propertyeditor.CalendarPropertyEditor;
import net.sf.springlayout.web.controller.propertyeditor.NullableDecimalPropertyEditor;
import net.sf.springlayout.web.layout.Form;
import net.sf.springlayout.web.message.Messages;
import net.sf.springlayout.web.security.PageAccessHolder;
import net.sf.springlayout.web.security.PageAccessManager;
import net.sf.springlayout.web.security.PageAccessResolver;
import net.sf.springlayout.web.util.RequestUtilSupport;
import net.sf.springlayout.web.util.WebConstants;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.propertyeditors.CharacterEditor;
import org.springframework.beans.propertyeditors.CustomBooleanEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.context.NoSuchMessageException;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;
import org.springframework.web.servlet.support.RequestContextUtils;

/**
 * <p>
 * Base form controller class for all controllers. Provides delegation to a
 * method in the controller based on the value of the "method" field / parameter
 * in the request.
 * </p>
 * <p>
 * Overrides the <code>referenceData()</code> method and delegates to
 * <code>setupReadModeReferenceData()</code> if the request is considered to
 * be in read mode or <code>setupEditModeReferenceData()</code> if the request
 * is in edit mode.
 * </p>
 * <p>
 * Also provides helper methods for getting localised messages and saving
 * messages to the session. Messages saved to the session should be used in
 * conjunction with an appropriate cleanup strategy such as the
 * UserResolvingInterceptor.
 * </p>
 * 
 * @see net.sf.springlayout.web.interceptor.MessagesInterceptor
 * @author Rob Monie, Adam Boas
 * 
 */
public abstract class AbstractBaseFormController extends SimpleFormController implements InitializingBean,
      FormController
{

   protected static Log log = LogFactory.getLog(AbstractBaseFormController.class);

   private Form form;

   private String defaultEditMode = WebConstants.EDIT_MODE_DEFAULT;

   protected static String PERSIST_VAL_THROUGH_REDIRECT = "persistValidateModeThroughRedirect";

   private boolean defaultPropertyEditorsAllowEmpty = false;

   private PageAccessResolver pageAccessResolver;

   private String redirectFormView;

   /**
    * Has this controller been set to allow nulls in property editors. By
    * default the basic property editors registered for binding by Spring will
    * throw binding errors if a null value is encountered. If this property is
    * set to true, all default property editors are registered to allow null.
    * 
    * @return
    */
   protected boolean isDefaultPropertyEditorsAllowEmpty()
   {
      return defaultPropertyEditorsAllowEmpty;
   }

   /**
    * Sets the pageAccessResolver for the controller
    * 
    * @param pageAccessResolver
    */
   public void setPageAccessResolver(PageAccessResolver pageAccessResolver)
   {
      this.pageAccessResolver = pageAccessResolver;
   }

   /**
    * Getes the pageAccessResolver for this request
    * 
    * @return
    */
   public PageAccessResolver getPageAccessResolver()
   {
      return pageAccessResolver;
   }

   /**
    * Set the controller to override the default Spring behaviour of registering
    * property editors with allow empty as false to allowEmpty=true If this
    * property is set to true, all default property editors are registered to
    * allow null values. default if not set is 'false' to maintain consistency
    * with default Spring behaviour.
    * 
    * @param defaultPropertyEditorsAllowEmpty
    */
   public void setDefaultPropertyEditorsAllowEmpty(boolean defaultPropertyEditorsAllowEmpty)
   {
      this.defaultPropertyEditorsAllowEmpty = defaultPropertyEditorsAllowEmpty;
   }

   /**
    * The form for this controller
    * 
    * @param form
    */
   public void setForm(Form form)
   {
      this.form = form;
   }

   /**
    * Sets the default edit mode - "1" or "true" for editMode, "0" or "false"
    * for read mode.
    * 
    * Default if not set is false
    * 
    * @param defaultEditMode
    *           The defaultEditMode to set.
    */
   public void setDefaultEditMode(String defaultEditMode)
   {
      if (defaultEditMode != null && ("true".equals(defaultEditMode) || "1".equals(defaultEditMode)))
      {
         this.defaultEditMode = "1";
      }
      else if (defaultEditMode != null && ("false".equals(defaultEditMode) || "0".equals(defaultEditMode)))
      {
         this.defaultEditMode = "0";
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.springframework.web.servlet.mvc.BaseCommandController#initBinder(javax.servlet.http.HttpServletRequest,
    *      org.springframework.web.bind.ServletRequestDataBinder)
    */
   public void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception
   {
      super.initBinder(request, binder);
   }

   /**
    * Allows external multi-panel validators to get the command associated with
    * this controller.
    * 
    * @param request
    * @return the command object which this controller is using.
    * @throws Exception
    */
   public Object getCommandObject(HttpServletRequest request)
   {
      Object obj = null;
      try
      {
         obj = this.formBackingObject(request);
      }
      catch (Exception e)
      {
         log.warn("Exception thrown in attempting to get command object from controller", e);
         if (e instanceof RuntimeException)
            throw (RuntimeException) e;
      }
      return obj;
   }

   /**
    * Gets the default edit mode
    * 
    * @return
    */
   public String getDefaultEditMode()
   {
      return defaultEditMode;
   }

   /**
    * Calls the method specified in the "method" parameter of the request if one
    * is available with a signature of: <methodName>(Httprequest request, Object
    * command) Whether this method is invoked or not it then calls
    * <code>onBindAfterPreValidationMethod</code>. Please note that any code
    * which would have been placed in the <code>onBind</code> method under the
    * normal Spring framework must now be placed in
    * <code>onBindAfterPreValidationMethod</code>.
    */
   protected final void onBind(HttpServletRequest request, Object command, BindException errors) throws Exception
   {
      String methodName = request.getParameter("method");
      if (methodName != null && !methodName.equals(""))
      {
         Class[] params = new Class[] { HttpServletRequest.class, Object.class, BindException.class };
         Object[] args = new Object[] { request, command, errors };
         Method method = this.getMethod(methodName, params);
         if (method != null)
         {
            try
            {
               method.invoke(this, args);
            }
            catch (InvocationTargetException e)
            {
               if (e.getCause() instanceof RuntimeException)
               {
                  throw (RuntimeException) e.getCause();
               }
               else
               {
                  throw new RuntimeException(e.getCause());
               }

            }
         }
         else
         {
            log.debug("No method defined to run onBind (method: " + methodName + "). Continuing processing.");
         }
      }

      onBindAfterPreValidationMethod(request, command, errors);
   }

   /**
    * Should be implemented by subclasses that wish to perform an action before
    * validation takes place.
    * 
    * @see org.springframework.web.servlet.mvc.BaseCommandController#onBind(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object)
    * @param request
    * @param command
    * @throws Exception
    */
   protected void onBindAfterPreValidationMethod(HttpServletRequest request, Object command, BindException errors)
         throws Exception
   {
   }

   /**
    * Calls the method specified in the "method" parameter of the request.
    * Enables process specific methods to be defined in the controller such as
    * <code>save()</code>, <code>delete()</code>, <code>approve()</code>,
    * <code>reject()</code>.
    * 
    * @see org.springframework.web.servlet.mvc.SimpleFormController#onSubmit(javax.servlet.http.HttpServletRequest,
    *      javax.servlet.http.HttpServletResponse, java.lang.Object,
    *      org.springframework.validation.BindException)
    */
   protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command,
         BindException errors) throws Exception
   {

      String methodName = request.getParameter("method");

      if (methodName == null || methodName.equals(""))
      {
         log.debug("No method specified for controller");
         throw new NoSuchControllerMethodException("No Method Specified", getClass());
      }
      try
      {
         Class[] params = new Class[] { HttpServletRequest.class, HttpServletResponse.class, Object.class,
               BindException.class };
         Object[] args = new Object[] { request, response, command, errors };
         Method method = this.getMethod(methodName, params);
         if (method == null)
            throw new NoSuchControllerMethodException(methodName, this.getClass());
         log.debug("Invoking " + methodName + " in " + this.getClass().getName());
         return (ModelAndView) method.invoke(this, args);
      }

      catch (InvocationTargetException e)
      {
         if (e.getCause() instanceof RuntimeException)
         {
            throw (RuntimeException) e.getCause();
         }
         else
         {
            throw new RuntimeException(e.getCause());
         }

      }
   }

   /**
    * Helper method to return the concrete method with the name and arguments
    * provided.
    * 
    * @param methodName
    *           for the method to retrieve.
    * @param args
    *           which the method must have in its signiture.
    * @return the method with the signiture provided.
    */
   private Method getMethod(String methodName, Class[] params)
   {
      try
      {

         Class clazz = this.getClass();
         Method method = clazz.getMethod(methodName, params);

         return method;
      }

      catch (NoSuchMethodException e)
      {
         return null;
      }
   }

   /**
    * Calls through to the <code>Form</code> validate method.. Subclasses that
    * override this method should call super if
    * <code>requestAwareValidator</code> or <code>PanelFormGroup</code>
    * validation is to be automatically invoked.
    * 
    * @see org.springframework.web.servlet.mvc.BaseCommandController#onBindAndValidate(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.BindException)
    */
   protected void onBindAndValidate(HttpServletRequest request, Object command, BindException errors) throws Exception
   {

      if (!this.suppressValidation(request))
      {
         if (this.form != null)
         {
            log.debug("Validating form");
            form.validate(request, command, errors);
         }
         else
         {
            log.debug("No Layout Form provided for controller.  No validation will be run on command object: " + this.getCommandName());
         }
      }
      else
      {
         log.debug("Validation suppressed");
      }

   }

   /**
    * Default implementation here first checks to see the mode and defers to
    * <code>setupReadModeReferenceData</code> or
    * <code>setupEditModeReferenceData</code> depending on the current mode.
    * 
    * <p>
    * Override either of these methods to get referencedata setup at the
    * appropriate time. If more control is required, override this method
    * directly and defer to <code>setupReadModeReferenceData</code> or
    * <code>setupEditModeReferenceData</code> when appropriate.
    * 
    * @see org.springframework.web.servlet.mvc.SimpleFormController#referenceData(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.Errors)
    */
   protected Map referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception
   {

      Map referenceData = new HashMap();

      if (this.isEditMode(request))
      {
         referenceData = setupEditModeReferenceData(request, command, errors);
      }
      else
      {
         referenceData = setupReadModeReferenceData(request, command, errors);
      }

      return referenceData;

   }

   /**
    * <p>
    * Create a reference data map in EDIT MODE for the given request and
    * command, consisting of bean name/bean instance pairs as expected by
    * ModelAndView.
    * </p>
    * <p>
    * Subclasses can override this to setup reference data used in the view.
    * Override this method instead of the standard referenceData() method to get
    * conditional execution based on the page read / edit state for free.
    * </p>
    * <p>
    * NOTE: This method will be called in EDIT mode only
    * </p>
    * 
    * @param request
    *           current HTTP request
    * @param command
    *           form object with request parameters bound onto it
    * @param errors
    *           validation errors holder
    * @return a Map with reference data entries, or <code>null</code> if none
    * @throws Exception
    *            in case of invalid state or arguments
    * @see org.springframework.web.servlet.mvc.SimpleFormController#referenceData(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.Errors)
    */
   protected Map setupEditModeReferenceData(HttpServletRequest request, Object command, Errors errors) throws Exception
   {
      return new HashMap();
   }

   /**
    * <p>
    * Create a reference data map in READ MODE for the given request and
    * command, consisting of bean name/bean instance pairs as expected by
    * ModelAndView.
    * </p>
    * <p>
    * Subclasses can override this to setup reference data used in the view.
    * Override this method instead of the standard referenceData() method to get
    * conditional execution based on the page read / edit state for free.
    * </p>
    * <p>
    * NOTE: This method will be called in READ mode only
    * </p>
    * 
    * @param request
    *           current HTTP request
    * @param command
    *           form object with request parameters bound onto it
    * @param errors
    *           validation errors holder
    * @return a Map with reference data entries, or <code>null</code> if none
    * @throws Exception
    *            in case of invalid state or arguments
    * @see org.springframework.web.servlet.mvc.SimpleFormController#referenceData(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.Errors)
    */
   protected Map setupReadModeReferenceData(HttpServletRequest request, Object command, Errors errors) throws Exception
   {
      return new HashMap();
   }

   /**
    * <p>
    * Adds a <code>PageAccessHolder</code> to the request using the
    * <code>WebConstants.PAGE_ACCESS_HOLDER</code> attribute with default
    * settings based on the <code>WebConstants.EDIT_MODE_PARAM</code> passed
    * with the request. Calls through to super to return to the regular
    * controller lifecycle. On GET's calls the
    * <code>PageAccessResolver</ciode> for the 
    * controller if present.
    * </p>
    * 
    * <p>
    * Note: If the <code>WebConstants.PAGE_ACCESS_HOLDER</code> attribute is
    * set at a lower level controller such as the AbstractListFormController,
    * the existing pageAccessHolder will remain untouched and the
    * <code>WebConstants.EDIT_MODE_PARAM</code> will be ignored.
    * 
    * @see org.springframework.web.servlet.mvc.AbstractController#handleRequestInternal(javax.servlet.http.HttpServletRequest,
    *      javax.servlet.http.HttpServletResponse)
    */
   protected ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response)
         throws Exception
   {

      PageAccessHolder pageAccessHolder = new PageAccessHolder();
      PageAccessManager.setPageAccessHolder(pageAccessHolder);

      if (this.isFormSubmission(request))
      {
         // always considered to be in edit mode on a post
         this.setEditMode(request, true);
      }
      else
      {
         this.setEditMode(request, RequestUtils.getBooleanParameter(request, WebConstants.EDIT_MODE_PARAM, "1"
               .equals(this.defaultEditMode)));
      }

      if (this.isEditMode(request))
      {
         pageAccessHolder.setReadOnlyFields(this.defineReadOnlyFields(request));
      }

      request.setAttribute(WebConstants.COMMAND_NAME, this.getCommandName());

      Map stateMap = this.getStateMap(request);

      stateMap.put(WebConstants.EDIT_MODE_PARAM, this.isEditMode(request) ? "1" : "0");

      if (this.isFormSubmission(request))
      {
         String commandDirty = RequestUtils.getStringParameter(request, WebConstants.COMMAND_DIRTY, "0");
         this.setStateMapField(request, WebConstants.COMMAND_DIRTY, commandDirty);

      }
      else
      {

         if (this.isEditMode(request) && stateMap.get(WebConstants.COMMAND_DIRTY) == null)
         {
            String commandDirty = (String) request.getSession().getAttribute(WebConstants.COMMAND_DIRTY);
            if (commandDirty == null)
            {
               commandDirty = "0";
            }
            this.setStateMapField(request, WebConstants.COMMAND_DIRTY, commandDirty);
            // cleanup the session arttribute
            request.getSession().removeAttribute(WebConstants.COMMAND_DIRTY);
         }

      }

      request.setAttribute(WebConstants.STATE_MAP, stateMap);

      ModelAndView mav = null;
      Object command = null;
      
      ServletRequestDataBinder binder = null;
      // Form submission or new form to show?
      if (isFormSubmission(request))
      {
         // Fetch form object from HTTP session, bind, validate, process submission.
         try
         {
            
            // Allow dataGrid users the opportunity to push a new object into their collections
            // before binding occurs.
            command = getCommand(request);
            this.editCommandBeforeBind(request, command);
            manageDataGrid(request);
            binder = bindAndValidate(request, command);
            mav = processFormSubmission(request, response, command, binder.getErrors());
         }
         catch (ServletException ex)
         {
            // Cannot submit a session form if no form object is in the
            // session.
            if (logger.isDebugEnabled())
            {
               logger.debug("Invalid submit detected: " + ex.getMessage());
            }
            mav = handleInvalidSubmit(request, response);
         }
      }

      else
      {
         // New form to show: render form view.
         mav = showNewForm(request, response);
         if (mav != null)
         {
            command = mav.getModel().get(this.getCommandName());
            binder = createBinder(request, command);
         }
      }
   
      // Always run on a GET. Run on a POST if explicitly requested.
      if (this.pageAccessResolver != null && command != null
            && (!this.isFormSubmission(request) || (this.isFormSubmission(request)
            && this.pageAccessResolver.isRunOnFormSubmission()) || binder.getErrors().hasErrors()))
      {
         this.pageAccessResolver.initialisePageAccess(request, command);
      }

      // Set session attribute to survive a redirect after main body of
      // controller has run in case commandDirty flag is changed
      String commandDirty = (String) this.getStateMap(request).get(WebConstants.COMMAND_DIRTY);
      if (this.isFormSubmission(request))
      {
         if (commandDirty != null)
         {
            request.getSession().setAttribute(WebConstants.COMMAND_DIRTY, commandDirty);
         }
      }
      else
      {
         // cleanup the session arttribute after it's been opened from a get
         request.getSession().removeAttribute(WebConstants.COMMAND_DIRTY);

         // fallback cleanup if validate mode param is hanging around from a
         // multiPanel controller
         // used on a previous post.
         if (!this.isPersistValidateModeThroughGetRedirect(request))
         {
            request.getSession().removeAttribute(WebConstants.VALIDATE_MODE_PARAM);
         }
      }

      return mav;
   }

   /**
    * Manages datagrid. Moves datagrid management fields into the request
    * attributes for use after a forward generated by errors.
    * 
    * @param request
    */
   protected void manageDataGrid(HttpServletRequest request)
   {

      String[] gridIds = RequestUtils.getStringParameters(request, "dataGridIds");

      for (int ii = 0; ii < gridIds.length; ii++)
      {

         request.setAttribute("dataGrid_addList_" + gridIds[ii], RequestUtils.getStringParameter(request,
               "dataGrid_addList_" + gridIds[ii], ""));
         request.setAttribute("dataGrid_removeList_" + gridIds[ii], RequestUtils.getStringParameter(request,
               "dataGrid_removeList_" + gridIds[ii], ""));
      }

   }

   /**
    * @param request
    */
   protected void editCommandBeforeBind(HttpServletRequest request, Object command)
   {
      return;
   }

   /**
    * A convenience method to provide an Array of indexes which the User has
    * requested to be removed from the collection contained in the DataGrid
    * identified by the gridId provided. This method is intended to be called
    * from <code>editCommandBeforeBind()</code> to allow the Collection which
    * the DataGrid is bound to to be modified before binding occurs.
    * 
    * @param request
    * @param gridId
    *           identifies the DataGrid which has 0 or more indexes to be
    *           removed.
    * @return Array of indeses which the user has requested to be removed from
    *         the Collection in the DataGrid identified by the gridId provided.
    * @throws ServletRequestBindingException
    */
   protected int[] getGridRemoveIndexes(HttpServletRequest request, String gridId)
         throws ServletRequestBindingException
   {
      String indexString = RequestUtils.getStringParameter(request, "dataGrid_removeList_" + gridId, null);
      if (indexString == null)
         return new int[0];

      String[] indexesAsStrings = StringUtils.delimitedListToStringArray(indexString, ",");
      int[] indexes = new int[indexesAsStrings.length];
      for (int ii = 0; ii < indexesAsStrings.length; ii++)
      {
         indexes[ii] = Integer.parseInt(indexesAsStrings[ii]);
      }
      return indexes;
   }

   /**
    * A convenience method to return the size of the list for the DataGrid
    * identified by the gridId provided.
    * 
    * @param request
    * @param gridId
    *           the id of the DataGrid to check for a user request to add a row.
    * @return the size of the list required for the grid to bind against or -1
    *         if list size is unchanged.
    */
   protected int getNewGridSize(HttpServletRequest request, String gridId)
   {
      try
      {
         String indexString = RequestUtils.getRequiredStringParameter(request, "dataGrid_addList_" + gridId);
         String[] indexes = indexString.split(",");
         int highestIndex = Integer.parseInt(indexes[indexes.length - 1]);
         return highestIndex + 1;
      }
      catch (ServletRequestBindingException e)
      {
         return -1;
      }
   }

   /**
    * Checks if the validatemode has been set to survive a redirect triggered by
    * a get.
    * 
    * @param request
    * @return
    */
   protected boolean isPersistValidateModeThroughGetRedirect(HttpServletRequest request)
   {
      String persistValidateMode = (String) request.getAttribute(PERSIST_VAL_THROUGH_REDIRECT);
      if (persistValidateMode != null && "1".equals(persistValidateMode))
      {
         return true;
      }
      else
      {
         return false;
      }
   }

   /**
    * Sets whether the current validation mode state should survive a redirect
    * caused by a get. This is useful if a condition is required on opening of a
    * GET request that may redirect to another page and the validation mode is
    * to be maintained.
    * 
    * @param request
    */
   protected void setPersistValidateModeThroughGetRedirect(HttpServletRequest request)
   {
      request.setAttribute(PERSIST_VAL_THROUGH_REDIRECT, "1");
   }

   /**
    * Gets a message from the resource bundle based on the current locale and
    * key / arguments passed.
    * 
    * @param code
    *           the key to the message
    * @param args
    *           arguments to be parsed with the message
    * @param request
    *           current request
    * @return a localised message
    * @throws NoSuchMessageException
    */
   protected String getMessage(String code, Object[] args, HttpServletRequest request) throws NoSuchMessageException
   {
      Locale locale = RequestContextUtils.getLocale(request);
      return this.getMessageSourceAccessor().getMessage(code, args, locale);
   }

   /**
    * Gets a message from the resource bundle based on the current locale and
    * key passed.
    * 
    * @param code
    *           the key to the message
    * @param request
    *           current request
    * @return a localised message
    * @throws NoSuchMessageException
    */
   protected String getMessage(String code, HttpServletRequest request) throws NoSuchMessageException
   {
      Locale locale = RequestContextUtils.getLocale(request);
      return this.getMessageSourceAccessor().getMessage(code, locale);
   }

   /**
    * Save the messages to the session
    * 
    * @param messages
    * @param request
    */
   protected void saveMessages(Messages messages, HttpServletRequest request)
   {
      request.getSession().setAttribute(WebConstants.MESSAGES, messages);
   }

   /**
    * Sets the edit mode for the current request.
    * 
    * @param request
    * @param editMode
    *           to set the current request to: true - editMode, false -
    *           readmode.
    */
   protected void setEditMode(HttpServletRequest request, boolean editMode)
   {

      PageAccessHolder pageAccessHolder = PageAccessManager.getPageAccessHolder();
      if (editMode)
      {
         log.debug("setting edit mode");
         pageAccessHolder.setEditMode();
         setStateMapField(request, WebConstants.EDIT_MODE_PARAM, "1");
      }
      else
      {
         log.debug("setting read mode");
         pageAccessHolder.setReadMode();
         setStateMapField(request, WebConstants.EDIT_MODE_PARAM, "0");
      }
   }

   /**
    * Retrieves whether or not the current request is in edit mode by checking
    * the page Access Holder.
    * 
    * @param request
    * @return true if in edit mode or false if in read mode.
    */
   private boolean isEditMode(HttpServletRequest request)
   {
      log.debug("checking for edit mode");
      PageAccessHolder pageAccessHolder = PageAccessManager.getPageAccessHolder();
      return pageAccessHolder.isEditMode();
   }

   /**
    * Calls through to secureFormBackingObject which is now deprecated.
    * 
    * @see org.springframework.web.servlet.mvc.AbstractFormController#formBackingObject(javax.servlet.http.HttpServletRequest)
    */
   protected Object formBackingObject(HttpServletRequest request) throws Exception
   {

      Object object = this.secureFormBackingObject(request);

      return object;
   }

   /**
    * 
    * @param request
    * @return
    * @throws Exception
    * @deprecated No longer need to override this method to get
    *             pageAccessResolver functionality.
    */
   protected Object secureFormBackingObject(HttpServletRequest request) throws Exception
   {
      return createCommand();
   }

   /**
    * Override to define a map of field paths that are to be treated as readOnly
    * when the page is in edit mode. This default implementation returns an
    * empty fieldMap
    * 
    * @param request
    * @return map of fields for read only display
    */
   public Map defineReadOnlyFields(HttpServletRequest request)
   {
      return new HashMap();
   }

   /**
    * Refreshes the current page after binding fields to the model. Useful for
    * generic refreshing of the UI after front end changes
    * 
    * @param request
    * @param response
    * @param command
    * @param errors
    * @return
    * @throws Exception
    */
   public ModelAndView refresh(HttpServletRequest request, HttpServletResponse response, Object command,
         BindException errors) throws Exception
   {

      log.debug("refreshing page");
      return this.showForm(request, response, errors);
   }

   /**
    * Overridden to allow registering of default property editor with default to
    * allow null.
    * 
    * All current spring behaviour has been maintained within this class. Class
    * has been completely overridden in order to slot default property editor
    * registering at the appropriate place
    * 
    * @param request
    *           current HTTP request
    * @param command
    *           the command to bind onto
    * @return the new binder instance
    * @throws Exception
    *            in case of invalid state or arguments
    * @see #bindAndValidate
    * @see #initBinder
    * @see #setMessageCodesResolver
    * @see #setBindingErrorProcessor
    */
   protected ServletRequestDataBinder createBinder(HttpServletRequest request, Object command) throws Exception
   {

      ServletRequestDataBinder binder = new LayoutServletRequestDataBinder(command, getCommandName());
      if (this.getMessageCodesResolver() != null)
      {
         binder.setMessageCodesResolver(this.getMessageCodesResolver());
      }
      if (this.getMessageCodesResolver() != null)
      {
         binder.setBindingErrorProcessor(this.getBindingErrorProcessor());
      }
      if (this.getPropertyEditorRegistrars() != null)
      {
         for (int i = 0; i < this.getPropertyEditorRegistrars().length; i++)
         {
            this.getPropertyEditorRegistrars()[i].registerCustomEditors(binder);
         }
      }

      if (defaultPropertyEditorsAllowEmpty)
      {
         registerDefaultPropertyEditors(request, binder);
      }

      initBinder(request, binder);
      return binder;
   }

   /**
    * register default property editors with allow empty
    * 
    * @param request
    * @param binder
    */
   protected void registerDefaultPropertyEditors(HttpServletRequest request, ServletRequestDataBinder binder)
   {
      log.debug("Registering default property editors with allow nulls set to true");
      // Default instances of character and boolean editors.
      // Can be overridden by registering custom instances of those as custom
      // editors.
      PropertyEditor characterEditor = new CharacterEditor(true);
      PropertyEditor booleanEditor = new CustomBooleanEditor(true);

      // The JDK does not contain a default editor for char!

      binder.registerCustomEditor(char.class, characterEditor);
      binder.registerCustomEditor(Character.class, characterEditor);

      // Spring's CustomBooleanEditor accepts more flag values than the JDK's
      // default editor.
      binder.registerCustomEditor(boolean.class, booleanEditor);
      binder.registerCustomEditor(Boolean.class, booleanEditor);

      // The JDK does not contain default editors for number wrapper types!
      // Override JDK primitive number editors with our own
      // CustomNumberEditor.
      PropertyEditor byteEditor = new NullableDecimalPropertyEditor(NullableDecimalPropertyEditor.BYTE_CONVERTER);
      PropertyEditor shortEditor = new NullableDecimalPropertyEditor(NullableDecimalPropertyEditor.SHORT_CONVERTER);
      PropertyEditor integerEditor = new NullableDecimalPropertyEditor(NullableDecimalPropertyEditor.INTEGER_CONVERTER);
      PropertyEditor longEditor = new NullableDecimalPropertyEditor(NullableDecimalPropertyEditor.LONG_CONVERTER);
      PropertyEditor floatEditor = new CustomNumberEditor(Float.class, true);
      PropertyEditor doubleEditor = new CustomNumberEditor(Double.class, true);

      binder.registerCustomEditor(byte.class, byteEditor);
      binder.registerCustomEditor(Byte.class, byteEditor);

      binder.registerCustomEditor(short.class, shortEditor);
      binder.registerCustomEditor(Short.class, shortEditor);

      binder.registerCustomEditor(int.class, integerEditor);
      binder.registerCustomEditor(Integer.class, integerEditor);

      binder.registerCustomEditor(long.class, longEditor);
      binder.registerCustomEditor(Long.class, longEditor);

      binder.registerCustomEditor(float.class, floatEditor);
      binder.registerCustomEditor(Float.class, floatEditor);

      binder.registerCustomEditor(double.class, doubleEditor);
      binder.registerCustomEditor(Double.class, doubleEditor);

      binder.registerCustomEditor(BigDecimal.class, new CustomNumberEditor(BigDecimal.class, true));
      binder.registerCustomEditor(BigInteger.class, new CustomNumberEditor(BigInteger.class, true));

      DateFormat dateFormat = DateFormatUtil.getLocalizedDateFormat(RequestContextUtils.getLocale(request));

      PropertyEditor calendarEditor = new CalendarPropertyEditor(dateFormat, true);
      binder.registerCustomEditor(Calendar.class, calendarEditor);

   }

   /**
    * Retrieves the request parameter map in it's current updated state. If
    * opening a new record returns a new empty param map.
    * 
    * @param request
    * @return the request parameter map in it's current updated state.
    */
   protected Map getStateMap(HttpServletRequest request)
   {
      Map stateMap = (Map) request.getAttribute(WebConstants.STATE_MAP);
      if (stateMap == null)
      {
         stateMap = new HashMap();
         request.setAttribute(WebConstants.STATE_MAP, stateMap);
      }
      return stateMap;
   }

   /**
    * Sets the parameter name/value pair in the current state map. This state
    * map will be persisted across posts.
    * 
    * @param request
    * @param name
    * @param value
    */
   protected void setStateMapField(HttpServletRequest request, String name, String value)
   {
      getStateMap(request).put(name, value);
   }

   public boolean isCommandDirty(HttpServletRequest request)
   {
      Map stateMap = this.getStateMap(request);
      return "1".equals(stateMap.get(WebConstants.COMMAND_DIRTY));
   }

   public void setCommandDirty(HttpServletRequest request)
   {
      Map stateMap = this.getStateMap(request);
      stateMap.put(WebConstants.COMMAND_DIRTY, "1");
   }

   public void setCommandClean(HttpServletRequest request)
   {
      Map stateMap = this.getStateMap(request);
      stateMap.put(WebConstants.COMMAND_DIRTY, "0");
   }

   /**
    * Gets the state map with parameters only to be included in the url.
    * VALIDATE_MODE_PARAM and COMMAND_DIRTY are removed so they are not
    * reflected in the url. Use this method in preference to the regular
    * getStateMap method when redirecting with url params
    * 
    * @param request
    * @return a state map for use in urls
    */
   protected Map getStateMapForUrl(HttpServletRequest request)
   {
      return RequestUtilSupport.getStateMapForUrl(this.getStateMap(request));
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
    */
   public void afterPropertiesSet() throws Exception
   {

   }

   /*
    * (non-Javadoc)
    * 
    * @see org.springframework.web.servlet.mvc.SimpleFormController#processFormSubmission(javax.servlet.http.HttpServletRequest,
    *      javax.servlet.http.HttpServletResponse, java.lang.Object,
    *      org.springframework.validation.BindException)
    */
   protected ModelAndView processFormSubmission(HttpServletRequest request, HttpServletResponse response,
         Object command, BindException errors) throws Exception
   {
      if (log.isDebugEnabled())
      {
         log.debug(errors);
      }
      return super.processFormSubmission(request, response, command, errors);
   }

   public void setRedirectFormView(String redirectView)
   {
      this.redirectFormView = redirectView;
   }

   public String getRedirectFormView()
   {
      return redirectFormView;
   }

   /**
    * Calls the controller pageAccessResolver if it exists and then
    * super.showForm
    * 
    * @see org.springframework.web.servlet.mvc.SimpleFormController#showForm(javax.servlet.http.HttpServletRequest,
    *      javax.servlet.http.HttpServletResponse,
    *      org.springframework.validation.BindException)
    */
   protected ModelAndView showForm(HttpServletRequest request, HttpServletResponse response, BindException errors)
         throws Exception
   {

      // call the pageAccessResolver on a post if there we are reopening from
      // a foprward (after validation failure for instance)
      if (this.isFormSubmission(request) && this.pageAccessResolver != null)
      {
         this.pageAccessResolver.initialisePageAccess(request, errors.getTarget());
      }

      return super.showForm(request, response, errors);
   }

}
