/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.controller;

import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import net.sf.springlayout.web.layout.Form;
import net.sf.springlayout.web.layout.PanelForm;
import net.sf.springlayout.web.layout.PanelFormGroup;
import net.sf.springlayout.web.util.WebConstants;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.util.Assert;
import org.springframework.validation.BindException;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;

/**
 * Abstract base class for providing common methods to controllers grouped in a
 * multipage / multi-controller configuration. Should be used as an alternative
 * to the WizardFormController when a different controller is required for each
 * page and Spring Layout validation and tags are to be used.
 * 
 * @author Rob Monie
 * 
 */
public abstract class AbstractPanelFormController extends AbstractBaseFormController implements InitializingBean
{

   protected static Log log = LogFactory.getLog(AbstractPanelFormController.class);
   private PanelFormGroup panelFormGroup;
   private String redirectFormView;

   /**
    * Sets the panelFormGroup for this controller
    * 
    * @param panelFormGroup
    */
   public void setPanelFormGroup(PanelFormGroup panelFormGroup)
   {
      this.panelFormGroup = panelFormGroup;
   }

   /**
    * gets the panel form group
    * 
    * @return
    */
   public PanelFormGroup getPanelFormGroup()
   {
      return panelFormGroup;
   }

   /**
    * Overrides the setting of forms and throws an error if set. Only a
    * panelFormGroup should be set within this controller context
    * 
    * @param form
    */
   public void setForm(Form form)
   {
      Assert.isNull(form,
            "Cannot specify a form in an AbstractPanelFormController.  Please specifiy a panelForm instead.");
   }

   /**
    * Used to change panels on a multi-panel controller after binding the
    * current form data to the command object.
    * 
    * @param request
    * @param response
    * @param command
    * @param errors
    * @return
    * @throws Exception
    */
   public ModelAndView changePanel(HttpServletRequest request, HttpServletResponse response, Object command,
         BindException errors) throws Exception
   {
      int targetPanelNo = RequestUtils.getIntParameter(request, WebConstants.TARGET_PANEL_PARAM, 0);
      PanelForm targetPanelForm = (PanelForm) panelFormGroup.getPanelForms().get(targetPanelNo);

      // we don't want the commandDirty paramater being represented in the url -
      // only as a hidden field
      Map urlArgsMap = this.getStateMapForUrl(request);

      // redirect to the new tab controller, passing the id of the current
      return new ModelAndView(targetPanelForm.getRedirectFormViewName(), urlArgsMap);

   }

   /**
    * Supresseses validation of requests if not in validate mode.
    * 
    * @param request
    *           the current request
    * 
    * @see org.springframework.web.servlet.mvc.SimpleFormController#suppressValidation(javax.servlet.http.HttpServletRequest)
    */

   protected boolean suppressValidation(HttpServletRequest request)
   {
      try
      {
         Boolean validate = RequestUtils.getBooleanParameter(request, WebConstants.VALIDATE_MODE_PARAM);

         if (validate == null)
         {
            log.debug("supressing validation...");
            return true;
         }
         else if (validate.booleanValue())
         {
            return false;
         }
         else
         {
            log.debug("supressing validation...");
            return true;
         }
      }
      catch (ServletRequestBindingException e)
      {
         log.debug("supressing validation...");
         return true;
      }

   }

   /**
    * Checks the session for errors and if present and in validation mode adds
    * them to the current errors object and removes them from the session. This
    * allows errors to persist when moving between tabs in edit mode.
    * 
    * @see org.springframework.web.servlet.mvc.SimpleFormController#showForm(javax.servlet.http.HttpServletRequest,
    *      javax.servlet.http.HttpServletResponse,
    *      org.springframework.validation.BindException)
    */
   protected ModelAndView showForm(HttpServletRequest request, HttpServletResponse response, BindException errors)
         throws Exception
   {
      HttpSession session = request.getSession();
      BindException oldErrors = (BindException) session.getAttribute(WebConstants.ERRORS);
      if (oldErrors != null)
      {
         if (this.isValidateMode(request))
            // This was changed to use addError, as addAllErrors has been
            // changed in spring 2.0
            for (Iterator iter = oldErrors.getAllErrors().iterator(); iter.hasNext();)
            {
               errors.addError((ObjectError) iter.next());
            }
         // errors.addAllErrors(oldErrors);
         session.removeAttribute(WebConstants.ERRORS);
      }
      return super.showForm(request, response, errors);
   }

   /**
    * Adds the commandName to the request using the
    * <code>WebConstants.COMMAND_NAME</code> attribute which is required by
    * the layout:panel tag then calls super to return to normal controller
    * lifecycle.
    * 
    * @see org.springframework.web.servlet.mvc.AbstractController#handleRequestInternal(javax.servlet.http.HttpServletRequest,
    *      javax.servlet.http.HttpServletResponse)
    */
   protected ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response)
         throws Exception
   {
      Map stateMap = this.getStateMap(request);

      if (this.isFormSubmission(request))
      {
         String validateMode = RequestUtils.getStringParameter(request, WebConstants.VALIDATE_MODE_PARAM,
               WebConstants.VALIDATION_MODE_DEFAULT);
         this.setStateMapField(request, WebConstants.VALIDATE_MODE_PARAM, validateMode);
      }
      else if (stateMap.get(WebConstants.VALIDATE_MODE_PARAM) == null)
      {
         String validateMode = (String) request.getSession().getAttribute(WebConstants.VALIDATE_MODE_PARAM);
         if (validateMode == null)
         {
            validateMode = WebConstants.VALIDATION_MODE_DEFAULT;
         }
         this.setStateMapField(request, WebConstants.VALIDATE_MODE_PARAM, validateMode);

      }

      ModelAndView mav = super.handleRequestInternal(request, response);

      // Set session attribute to survive a redirect after main body of
      // controller has run in case validate mode flag is changed
      String validateMode = (String) this.getStateMap(request).get(WebConstants.VALIDATE_MODE_PARAM);
      if (this.isFormSubmission(request))
      {
         if (validateMode != null)
         {
            request.getSession().setAttribute(WebConstants.VALIDATE_MODE_PARAM, validateMode);
         }
      }
      else
      {
         if (!this.isPersistValidateModeThroughGetRedirect(request))
         {
            // cleanup the session attribute after it's been opened from a get
            request.getSession().removeAttribute(WebConstants.VALIDATE_MODE_PARAM);
         }
      }
      return mav;
   }

   /**
    * Refreshes the current page after binding fields to the model. Useful for
    * generic refreshing of the UI after front end changes
    * 
    * @param request
    * @param response
    * @param command
    * @param errors
    * @return
    * @throws Exception
    */
   public ModelAndView refresh(HttpServletRequest request, HttpServletResponse response, Object command,
         BindException errors) throws Exception
   {

      log.debug("refreshing page...");
      return new ModelAndView(this.getRedirectFormView(), this.getStateMapForUrl(request));
   }

   /**
    * Calls through to the <code>PanelFormGroup</code> validate method if
    * supplied before passing onto super...
    * 
    * @see org.springframework.web.servlet.mvc.BaseCommandController#onBindAndValidate(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.BindException)
    */
   protected void onBindAndValidate(HttpServletRequest request, Object command, BindException errors) throws Exception
   {

      if (!this.suppressValidation(request))
      {

         if (this.panelFormGroup != null)
         {
            this.setCurrentlyValidating(request, true);
            panelFormGroup.validate(request, command, errors);
            this.setCurrentlyValidating(request, false);
         }
      }

      super.onBindAndValidate(request, command, errors);
   }

   public void setRedirectFormView(String redirectFormView)
   {
      this.redirectFormView = redirectFormView;
   }

   public String getRedirectFormView()
   {
      return redirectFormView;
   }

   // TODO:this is just a test it must be removed an made more robust before
   // production(AB)
   /*
    * (non-Javadoc)
    * 
    * @see org.springframework.web.servlet.mvc.SimpleFormController#processFormSubmission(javax.servlet.http.HttpServletRequest,
    *      javax.servlet.http.HttpServletResponse, java.lang.Object,
    *      org.springframework.validation.BindException)
    */
   protected ModelAndView processFormSubmission(HttpServletRequest request, HttpServletResponse response,
         Object command, BindException errors) throws Exception
   {
      if (!isValidateMode(request))
      {
         return super.processFormSubmission(request, response, command, errors);
      }
      else
      {
         request.getSession().setAttribute(WebConstants.ERRORS, errors);
         if (isFormChangeRequest(request))
         {
            logger.debug("Detected form change request -> routing request to onFormChange");
            onFormChange(request, response, command, errors);
            return showForm(request, response, errors);
         }
         else
         {
            return onSubmit(request, response, command, errors);
         }
      }
   }

   protected boolean isValidateMode(HttpServletRequest request)
   {
      Map stateMap = this.getStateMap(request);
      return "1".equals(stateMap.get(WebConstants.VALIDATE_MODE_PARAM));
   }

   protected void setValidateMode(HttpServletRequest request, boolean validateMode)
   {
      Map stateMap = this.getStateMap(request);
      if (validateMode)
         stateMap.put(WebConstants.VALIDATE_MODE_PARAM, "1");
      else
         stateMap.put(WebConstants.VALIDATE_MODE_PARAM, "0");
   }

   /**
    * Sets a request attribute as to whether we're currently validating. This is
    * different to the validateMode param and the two should not be confused.
    * 
    * @param request
    * @param validating
    */
   protected void setCurrentlyValidating(HttpServletRequest request, boolean validating)
   {
      request.setAttribute(WebConstants.CURRENTLY_VALIDATING, new Boolean(validating));
   }

   /**
    * returns whether currently validating. Not to be confused with the
    * ValidateMode param.
    * 
    * @param request
    * @return
    */
   public boolean isCurrentlyValidating(HttpServletRequest request)
   {
      Boolean validating = (Boolean) request.getAttribute(WebConstants.CURRENTLY_VALIDATING);
      if (validating == null)
      {
         return false;
      }
      else
      {
         return validating.booleanValue();
      }

   }

   /**
    * Simple method to quickly add a parameter to the state map. Anything put in
    * the statemap will be reflected in the url throughout navigation of a
    * multi- panel form
    * 
    * @param request
    * @param key
    * @param value
    */
   public void addToStateMap(HttpServletRequest request, Object key, Object value)
   {
      this.getStateMap(request).put(key, value);
   }

   /**
    * Overridden to increase the visibility. Calls super.initBinder(request,
    * binder)
    * 
    * @see org.springframework.web.servlet.mvc.BaseCommandController#initBinder(javax.servlet.http.HttpServletRequest,
    *      org.springframework.web.bind.ServletRequestDataBinder)
    */
   public void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception
   {
      super.initBinder(request, binder);
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
    */
   public void afterPropertiesSet() throws Exception
   {

   }

}
