/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.springlayout.web.controller.propertyeditor;

import java.beans.PropertyEditorSupport;

import net.sf.springlayout.web.security.AccessLevel;
import net.sf.springlayout.web.security.PageAccessManager;


/**
 * <p>
 * Abstract class for creating security or mode aware property editors. To be
 * used in conjuntion with <code>PageAccessResolver</code>'s to provide correct values for
 * field read / edit modes.
 * </p>
 * <p>
 * Can determine the correct display mode based on the user's access to the page and / or field.
 * </p>
 * 
 * @author Rob Monie
 */
public abstract class AbstractSecurityAwarePropertyEditor extends PropertyEditorSupport
{
   private String fieldPath;
   private AccessLevel accessLevelOverride;

   /**
    * @param fieldPath
    *           the path to the field. used to determine the fields display mode
    */
   public AbstractSecurityAwarePropertyEditor()
   {
      super();
   }

   /**
    * @param fieldPath
    *           the path to the field. used to determine the fields display mode
    */
   public AbstractSecurityAwarePropertyEditor(String fieldPath)
   {
      super();
      this.fieldPath = fieldPath;
   }

   /**
    * @param fieldPath
    *           the path to the field. used to determine the fields display mode
    * @param accessLevelOverride
    *           if provided this will override any other determined display mode
    *           for the field
    */
   public AbstractSecurityAwarePropertyEditor(String fieldPath, AccessLevel accessLevelOverride)
   {
      super();
      this.fieldPath = fieldPath;
      this.accessLevelOverride = accessLevelOverride;
   }
   
   /**
    * @param accessLevelOverride
    *           if provided this will override any other determined display mode
    *           for the field
    */
   public AbstractSecurityAwarePropertyEditor(AccessLevel accessLevelOverride)
   {
      super();
      this.accessLevelOverride = accessLevelOverride;
   }

   /**
    * 
    * Determines correct mode for display and calls through to relevent method to resolve the display value 
    * @see java.beans.PropertyEditorSupport#getAsText()
    */
   public final String getAsText()
   {

      boolean editMode = false;
      if (fieldPath != null)
      {
         editMode = !PageAccessManager.getPageAccessHolder().isFieldReadOnly(this.fieldPath);
      }
      else
      {
         editMode = PageAccessManager.getPageAccessHolder().isEditMode();
      }

      if (getValue() == null)
      {
         return "";
      }
      else if ((editMode && accessLevelOverride != AccessLevel.READ_ACCESS)
            || accessLevelOverride == AccessLevel.EDIT_ACCESS)
      {
         return this.getAsTextEditMode();
      }
      else
      {
         return this.getAsTextReadMode();
      }
   }

   /**
    * @return value for edit mode
    */
   protected abstract String getAsTextEditMode();

   /**
    * @return value for read mode
    */
   protected abstract String getAsTextReadMode();

}
