/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import net.sf.springlayout.web.util.FieldPathUtils;
import net.sf.springlayout.web.util.WebConstants;
import net.sf.springlayout.web.validator.MaxLengthValidator;
import net.sf.springlayout.web.validator.NestedPathResolver;
import net.sf.springlayout.web.validator.ValidationRule;
import net.sf.springlayout.web.validator.condition.Condition;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.InvalidPropertyException;
import org.springframework.context.ApplicationContext;
import org.springframework.util.StringUtils;
import org.springframework.validation.Errors;
import org.springframework.web.servlet.support.RequestContextUtils;


/**
 * Defines a field for a panel providing information such as it's name, whether
 * it's mandatory and an optional array of validation rules. Validation is
 * called via the LayoutPanelFormGroup from controller.
 * 
 * @author Rob Monie, Adam Boas
 * 
 */
public class LayoutFieldDefinition implements FieldDefinition
{

   private static Log log = LogFactory.getLog(LayoutFieldDefinition.class);

   private String fieldKey;
   private List alternateFieldKeysForError;
   private boolean mandatory = false;
   private Condition mandatoryCondition;
   private List validationRules;
   

   public LayoutFieldDefinition()
   {

      this.validationRules = new ArrayList();
      this.alternateFieldKeysForError = new ArrayList();
   }

   public LayoutFieldDefinition(FieldDefinition defaultField)
   {
      this.fieldKey = defaultField.getFieldKey();
      this.alternateFieldKeysForError = defaultField.getAlternateFieldKeysForError();
      this.mandatory = defaultField.isMandatory();
      this.mandatoryCondition = defaultField.getMandatoryCondition();
      this.validationRules = defaultField.getValidationRules();

  
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IFieldDefinition#getFieldKey()
    */
   public String getFieldKey()
   {
      return fieldKey;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IFieldDefinition#setFieldKey(java.lang.String)
    */
   public void setFieldKey(String fieldKey)
   {
      this.fieldKey = fieldKey;
   }
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.FieldDefinition#isAlwaysMandatory()
    */
   public boolean isMandatory()
   {
      return this.mandatory;
   }
   
   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IFieldDefinition#setMandatory(boolean)
    */
   public void setMandatory(boolean mandatory)
   {
      this.mandatory = mandatory;
   }
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.FieldDefinition#setMandatoryCondition(net.sf.springlayout.web.validator.condition.Condition)
    */
   public void setMandatoryCondition(Condition mandatoryCondition)
   {
      this.mandatoryCondition = mandatoryCondition;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.FieldDefinition#getMandatoryConditions()
    */
   public Condition getMandatoryCondition()
   {
      return this.mandatoryCondition;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IFieldDefinition#getValidationRules()
    */
   public List getValidationRules()
   {
      return validationRules;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IFieldDefinition#setValidationRules(net.sf.springlayout.web.validator.ValidationRule[])
    */
   public void setValidationRules(List validationRules)
   {
      this.validationRules = validationRules;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.FieldDefinition#getDependentValidationFieldKeys()
    */
   public List getAlternateFieldKeysForError()
   {
      return alternateFieldKeysForError;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.FieldDefinition#setDependentValidationFieldKeys(java.util.List)
    */
   public void setAlternateFieldKeysForError(List dependentValidationFieldKeys)
   {
      this.alternateFieldKeysForError = dependentValidationFieldKeys;
   }

  
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.FieldDefinition#getMaxLength()
    */
   public int getMaxLength()
   {
      for (Iterator iter = this.getValidationRules().iterator();iter.hasNext();)
      {
         ValidationRule validationRule = (ValidationRule) iter.next();
         if (validationRule instanceof MaxLengthValidator)
            return ((MaxLengthValidator)validationRule).getMaxLength();
      }
      return -1;
   }
   
   /**
    * Validates a field based on whether it is mandatory, 
    * and then iterates through any custom validators defined for the
    * field. Validation occurs on all possible concrete paths represented by the field definition
    * 
    * @param request
    * @param obj
    * @param errors
    * 
    * @see net.sf.springlayout.web.panel.FieldDefinition#validate(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.Errors)
    */
   public void validate(HttpServletRequest request, 
                        NestedPathResolver resolver, 
                        BeanWrapper wrapper,
                        Errors errors)
   {

      if (WebConstants.DEFAULT_FIELD_NAME.equals(this.fieldKey)) 
      {
         //don't do anything if this is the default field
         return;
      }
      log.debug("validating field: " + this.fieldKey);

      List paths = resolver.getValidPaths(this.fieldKey);

      for (Iterator it = paths.iterator(); it.hasNext();)
      {
         String fieldId = (String) it.next();
         
         if (isFailedMandatory(wrapper, fieldId))
         {
            errors.rejectValue(fieldId, "errors.required", new Object[] { this.getFieldLabel(request, fieldId, null) },
                  "{0} is required");
            if (this.getAlternateFieldKeysForError() != null)
            {
               for (Iterator dependentIterator = this.getAlternateFieldKeysForError().iterator(); dependentIterator
                     .hasNext();)
               {
                  String correctDependentPath = FieldPathUtils.correctDependentPath(fieldId, (String) dependentIterator
                        .next());
                  errors.rejectValue(correctDependentPath, "errors.required", new Object[] { this.getFieldLabel(request,
                        correctDependentPath, null) }, "{0} is required");
               }
            }
         }

         List validationRules = this.getValidationRules();
         if (validationRules != null)
         {
            for (int ii = 0; ii < validationRules.size(); ii++)
            {
               ((ValidationRule) validationRules.get(ii)).conditionallyValidate(request, wrapper.getWrappedInstance(), errors, fieldId);
            }
         }
         
      }

   }
   
   public boolean isFailedMandatory(BeanWrapper wrappedObject, String fieldId)
   {
      Object value = null;
      try
      {
         value = wrappedObject.getPropertyValue(fieldId);
      }
      catch (InvalidPropertyException e)
      {
         // This could happen
      }
      catch (IndexOutOfBoundsException e)
      {
         // This could happen
      }
      if (this.isMandatory(wrappedObject, fieldId))
         return value == null || !StringUtils.hasText(value.toString());
      return false;
   }
   
   /**
    * is the current field status mandatory
    * 
    * @return
    */
   public boolean isMandatory(BeanWrapper wrappedObject, String fieldId)
   {
      return (this.isMandatory() || (this.getMandatoryCondition() != null && this.getMandatoryCondition().evaluate(fieldId, wrappedObject)));
   }
   
   /**
    * Gets a field label message based on the current locale and the field path
    * passed in. Prefixes the path with the command object name to find errors
    * based on the full path as defined in labels.properties files.
    * 
    * @param request
    *           the current request
    * @param code
    *           the message key
    * @return a localised message
    */
   private String getFieldLabel(HttpServletRequest request, String code, Object[] args)
   {
      Locale locale = RequestContextUtils.getLocale(request);
      ApplicationContext ctx = RequestContextUtils.getWebApplicationContext(request);
      StringBuffer path = new StringBuffer();
      path.append((String) request.getAttribute(WebConstants.COMMAND_NAME));
      if (!"".equals(path.toString()))
      {
         path.append(".");
      }
      path.append(code);
      String strippedPath = LayoutUtils.getStrippedPropertyPath(path.toString());
      String message = ctx.getMessage(strippedPath, args, locale);
      return message;
   }
}
