/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;

import net.sf.springlayout.web.util.WebConstants;
import net.sf.springlayout.web.validator.NestedPathResolver;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.validation.Errors;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;


/**
 * Default implementation of a Form
 * @author Rob Monie
 * 
 */
public class LayoutForm implements Form, InitializingBean
{

   private static Log log = LogFactory.getLog(LayoutForm.class);
   protected Map fieldDefinitions = new HashMap();
   protected String formName;
   protected String formAction;
   protected String controllerBeanName;

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IPanel#getFieldDefinitions()
    */
   public Map getFieldDefinitions()
   {
      return fieldDefinitions;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IPanel#setFieldDefinitions(java.util.Map)
    */
   public void setFieldDefinitions(Map fieldDefinitions)
   {
      this.fieldDefinitions = fieldDefinitions;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.PanelForm#getFieldDefinition(java.lang.String)
    * 
    * looks up the field definition requested, but if it does not exist then it
    * looks up the default field definition and puts that back in the map
    */
   public FieldDefinition getFieldDefinition(String path)
   {
      String strippedPath = LayoutUtils.getStrippedPropertyPath(path);

      FieldDefinition fieldDefinition = (FieldDefinition) fieldDefinitions.get(strippedPath);
      if (fieldDefinition == null)
      {
         fieldDefinition = (FieldDefinition) fieldDefinitions.get(WebConstants.DEFAULT_FIELD_NAME);
         if (fieldDefinition != null)
         {
            fieldDefinition = new LayoutFieldDefinition(fieldDefinition);
            fieldDefinition.setFieldKey(strippedPath);
            if (log.isDebugEnabled())
               log.debug("form " + formName + ". BeanName: " + controllerBeanName + " Using default field definition for " + LayoutUtils.getStrippedPropertyPath(path));
            fieldDefinitions.put(LayoutUtils.getStrippedPropertyPath(path), fieldDefinition);
         }
      }
      return fieldDefinition;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IPanel#validate(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.Errors)
    */
   public void validate(HttpServletRequest request, Object obj, Errors errors)
   {

      if (this.fieldDefinitions != null)
      {
         BeanWrapper wrapper = new BeanWrapperImpl(obj);
         NestedPathResolver resolver = new NestedPathResolver(obj);
         for (Iterator it = fieldDefinitions.entrySet().iterator(); it.hasNext();)
         {
            FieldDefinition fieldDefinition = (FieldDefinition) ((Map.Entry) it.next()).getValue();
            fieldDefinition.validate(request, resolver, wrapper, errors);
         }
      }

   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.panel.IPanel#getErrors(javax.servlet.http.HttpServletRequest,
    *      java.lang.String)
    */
   public Map getErrors(Errors errors)
   {
      Map panelErrors = new HashMap();

      log.debug("getting errors for form ");

      if (fieldDefinitions != null && fieldDefinitions.size() > 0 && errors.hasErrors())
      {
         for (Iterator it = errors.getAllErrors().iterator(); it.hasNext();)
         {
            ObjectError error = (ObjectError) it.next();
            if (error instanceof FieldError)
            {
               if (fieldDefinitions.get(LayoutUtils.getStrippedPropertyPath(((FieldError) error).getField())) != null)
               {
                  panelErrors.put(((FieldError) error).getField(), error);
               }

            }
         }
      }
      return panelErrors;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.PanelForm#hasErrors(org.springframework.validation.Errors)
    */
   public boolean hasErrors(Errors errors)
   {
      if (getErrors(errors).isEmpty())
      {
         log.debug("no errors found for form");

         return false;
      }
      else
      {
         if (log.isDebugEnabled())
         {
            log.debug(errors.getAllErrors().size() + " error(s) found for panel");
         }
         return true;
      }
   }

   /**
    * Test to ensure that a controller has been provided to the class and all
    * field definitions keys match the map key.
    */
   public void afterPropertiesSet()
   {

      if (this.fieldDefinitions != null)
      {
         StringBuffer errors= new StringBuffer();
         for (Iterator it = this.fieldDefinitions.entrySet().iterator(); it.hasNext();)
         {
            Entry entry = (Map.Entry) it.next();

            FieldDefinition fieldDefinition = (FieldDefinition) entry.getValue();

            String fieldKey = (String) entry.getKey();

            if (!fieldKey.equals(fieldDefinition.getFieldKey()))
            {
               throw new LayoutFormConfigurationException("Field definition with map key of '" + fieldKey
                     + " is not equal to its Field Definition key of '" + fieldDefinition.getFieldKey() + "'");
            }

            if (fieldDefinition.getMandatoryCondition() != null)
            {
               try
               {
                  fieldDefinition.getMandatoryCondition().afterPropertiesSet();
               }
               catch (LayoutFormConfigurationException e)
               {
                  errors.append(errors.length()>0 ? "":",\n");
                  errors.append(fieldDefinition.getFieldKey()+ " definition contains the following errors:" + e.getMessage());
               }
            }
         }
         if (errors.length() > 0)
            throw new LayoutFormConfigurationException(this.formName
                  + " contains configuration errors: \n" + errors.toString());

      }
      log.debug(this.formName + " loaded.");

   }

   public String getFormName()
   {
      return formName;
   }

   public void setFormName(String formName)
   {
      this.formName = formName;
   }

   public String getFormAction()
   {
      return formAction;
   }

   public void setFormAction(String formAction)
   {
      this.formAction = formAction;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.Form#getControllerBeanName()
    */
   public String getControllerBeanName()
   {
      return controllerBeanName;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.Form#setControllerBeanName(java.lang.String)
    */
   public void setControllerBeanName(String controllerBeanName)
   {
      this.controllerBeanName = controllerBeanName;
   }

}
