/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sf.springlayout.web.security.PageAccessHolder;
import net.sf.springlayout.web.util.FieldPathUtils;
import net.sf.springlayout.web.validator.ValidationRule;
import net.sf.springlayout.web.validator.condition.Condition;

import org.springframework.beans.BeanWrapper;
import org.springframework.beans.InvalidPropertyException;
import org.springframework.util.StringUtils;

/**
 * This is an instance of an actual field that is rendered on a page. There may
 * be multiple instances for the same fieldDefinition. Each with an indexed path
 * to the actual location within the command object. Most times there will at
 * least be a label and a field, both being separate
 * <code>OnPageFieldInstance</code>s.
 * 
 * The OnPageFieldInstance contains details about a field, it's actual id within
 * the form where it is to be rendered, and whether it is currently marked as a
 * mandatory field. This class allows a form to render aggregated Javascript at
 * the end of the form rendering process.
 * 
 * @author Adam Boas, Rob Monie
 * Created on 19/05/2006
 */
public class OnPageFieldInstance
{
   private String fieldId;
   private FieldDefinition fieldDefinition;
   private List externalManadatoryRules;

   private boolean readOnly = false;
   private boolean hasErrors = false;
   private boolean grouped = false;
   private int groupSize = 1;

   /** default constructor */
   public OnPageFieldInstance()
   {
      this.externalManadatoryRules = new ArrayList();
   }

   /**
    * @param fieldId
    * @param fieldDefinition
    * @param currentlyMandatory
    */
   public OnPageFieldInstance(String fieldId, FieldDefinition fieldDefinition, PageAccessHolder pageAccessHolder,
         boolean grouped, int groupSize)
   {
      this.fieldId = fieldId;
      this.fieldDefinition = fieldDefinition;
      this.externalManadatoryRules = new ArrayList();
      this.grouped = grouped;
      this.groupSize = groupSize;
      setReadOnly(pageAccessHolder.isFieldReadOnly(fieldId));

   }

   /**
    * Retrieves the externalManadatoryRules for this OnPageFieldInstance.
    * 
    * @return Returns the externalManadatoryRules for this OnPageFieldInstance.
    */
   public List getExternalManadatoryRules()
   {
      return externalManadatoryRules;
   }

   /**
    * Sets the externalManadatoryRules for this OnPageFieldInstance.
    * 
    * @param externalManadatoryRules
    *           Sets the externalManadatoryRules for this OnPageFieldInstance.
    */
   public void setExternalManadatoryRules(List externalManadatoryRules)
   {
      this.externalManadatoryRules = externalManadatoryRules;
   }

   /**
    * @param rule
    */
   public void addExternalMandatoryRule(String rule)
   {
      this.externalManadatoryRules.add(rule);
   }

   /**
    * Retrieves the fieldDefinition for this OnPageFieldInstance.
    * 
    * @return Returns the fieldDefinition for this OnPageFieldInstance.
    */
   public FieldDefinition getFieldDefinition()
   {
      return fieldDefinition;
   }

   /**
    * Sets the fieldDefinition for this OnPageFieldInstance.
    * 
    * @param fieldDefinition
    *           Sets the fieldDefinition for this OnPageFieldInstance.
    */
   public void setFieldDefinition(FieldDefinition fieldDefinition)
   {
      this.fieldDefinition = fieldDefinition;
   }

   /**
    * Retrieves the fieldId for this OnPageFieldInstance.
    * 
    * @return Returns the fieldId for this OnPageFieldInstance.
    */
   public String getFieldId()
   {
      return fieldId;
   }

   /**
    * Sets the fieldId for this OnPageFieldInstance.
    * 
    * @param fieldId
    *           Sets the fieldId for this OnPageFieldInstance.
    */
   public void setFieldId(String fieldId)
   {
      this.fieldId = fieldId;
   }

   /**
    * is the current field status mandatory
    * 
    * @return
    */
   public boolean isMandatory(BeanWrapper wrappedObject)
   {
      return (fieldDefinition.isMandatory() || (fieldDefinition.getMandatoryCondition() != null && fieldDefinition
            .getMandatoryCondition().evaluate(this.getFieldId(), wrappedObject)));
   }

   /**
    * get the current field readonly property
    * 
    * @return
    */
   public boolean getReadOnly()
   {
      return readOnly;
   }

   /**
    * set the current field readonly property
    * 
    * @param readOnly
    */
   public void setReadOnly(boolean readOnly)
   {
      this.readOnly = readOnly;
   }

   /**
    * Get the current error status for the field
    * 
    * @return
    */
   public boolean getHasErrors()
   {
      return hasErrors;
   }

   /**
    * set the current error status for the field
    * 
    * @param hasErrors
    */
   public void setHasErrors(boolean hasErrors)
   {
      this.hasErrors = hasErrors;
   }

   public String getJavascript(boolean isValidationMode, Object command, String formName)
   {
      if(this.readOnly)
         return "";
      
      Condition condition = this.fieldDefinition.getMandatoryCondition();
      StringBuffer script = new StringBuffer();
      StringBuffer postProcessingScript = new StringBuffer();

      String groupedIndex = "";

      if (grouped)
      {
         groupedIndex = "[0]";
      }

      script.append("document.forms['");
      script.append(formName);
      script.append("']['");
      script.append(this.fieldId);
      // register only against the first field as the master for better performance
      script.append("']" + groupedIndex + ".isMandatory = function(field){");
      if (fieldDefinition.isMandatory())
      {
         script.append("return true;");
      }
      else if (fieldDefinition.getMandatoryCondition() == null)
      {
         script.append("return false;");
      }
      else
      {
         script.append("return ");
         script.append(condition.getJavascript(this, command, formName));
         script.append(";");
         postProcessingScript.append(condition.getPostProcessingJavascript(this, formName));

      }

      postProcessingScript.append("document.forms['");
      postProcessingScript.append(formName);
      postProcessingScript.append("']['");
      postProcessingScript.append(this.fieldId);
      // register only against the first field as the master for better
      // performance
      postProcessingScript.append("']" + groupedIndex + ".fieldLabelsForError = [");

      if (this.fieldDefinition.getAlternateFieldKeysForError().size() > 0)
      {
         for (Iterator it = this.fieldDefinition.getAlternateFieldKeysForError().iterator(); it.hasNext();)
         {
            String actualLabelId = FieldPathUtils.correctDependentPath(fieldId, StringUtils.trimWhitespace(it.next()
                  .toString())
                  + "_label");
            postProcessingScript.append("\"" + actualLabelId + "\"" + (it.hasNext() ? "," : ""));
         }
      }
      else
      {
         String actualLabelId = fieldId + "_label";
         postProcessingScript.append("\"" + actualLabelId + "\"");
      }

      postProcessingScript.append("];\n");

      script.append("};\n");

      // Generate validation javascript
      script.append("document.forms['");
      script.append(formName);
      script.append("']['");
      script.append(this.fieldId);

      script.append("']" + groupedIndex + ".isValid = function(field, alerts){\n ");
      script.append("var mandatoryResult = true;\n");
      script.append("var validationResult = true;\n");

      if (isValidationMode)
      {
         script.append("if(field.isMandatory(field)){mandatoryResult = validateMandatory(field);}\n");
      }

      if (fieldDefinition.getValidationRules().size() > 0)
      {
         script.append("var checkValidationRules  = function(field, alerts) {\n");

         String ruleResult = "";
         int ruleIndex = 0;
         for (Iterator it = this.fieldDefinition.getValidationRules().iterator(); it.hasNext();)
         {

            script.append("var rule");
            script.append(ruleIndex);
            script.append(" = ");
            ValidationRule validationRule = (ValidationRule) it.next();
            script.append(validationRule.getConditionalJavascript(this, command, formName));
            ruleResult += ruleIndex == 0 ? "rule" + ruleIndex : " && rule" + ruleIndex;
            ruleIndex ++;
         }

         script.append("\n return ");
         script.append(ruleResult);
         script.append(";};\n");

         script.append("return mandatoryResult && checkValidationRules(field, alerts)");

      }
      else
      {
         script.append("return mandatoryResult");
      }
      script.append(";};\n");

      script.append(postProcessingScript);
      return script.toString();
   }

   /**
    * @return
    */
   public boolean isFailedMandatory(BeanWrapper wrappedObject)
   {
      Object value = null;
      try
      {
         value = wrappedObject.getPropertyValue(fieldId);
      }
      catch (InvalidPropertyException e)
      {
         // This could happen
      }
      catch (IndexOutOfBoundsException e)
      {
         // This could happen
      }
      if (this.isMandatory(wrappedObject))
         return value == null || !StringUtils.hasText(value.toString());
      return false;
   }

   /**
    * Validates the field for mandatory'ness and valid data
    * 
    * @param wrappedObject
    * @return
    */
   public boolean isFailedValidation(BeanWrapper wrappedObject)
   {
      if (isFailedMandatory(wrappedObject))
         return true;
      List validationRules = fieldDefinition.getValidationRules();
      if (validationRules != null)
      {
         for (int ii = 0; ii < validationRules.size(); ii++)
         {
            ValidationRule validationRule = (ValidationRule) validationRules.get(ii);
            if (!validationRule.isValid(wrappedObject.getWrappedInstance(), fieldId))
            {
               return true;
            }
         }
      }
      return false;

   }

   /**
    * @return the isGrouped
    */
   public boolean isGrouped()
   {
      return grouped;
   }

   /*
    * (non-Javadoc)
    * 
    * @see java.lang.Object#hashCode()
    */
   public int hashCode()
   {
      final int PRIME = 31;
      int result = 1;
      result = PRIME * result + ((fieldId == null) ? 0 : fieldId.hashCode());
      return result;
   }

   /*
    * (non-Javadoc)
    * 
    * @see java.lang.Object#equals(java.lang.Object)
    */
   public boolean equals(Object obj)
   {
      if (this == obj)
         return true;
      if (obj == null)
         return false;
      if (getClass() != obj.getClass())
         return false;
      final OnPageFieldInstance other = (OnPageFieldInstance) obj;
      if (fieldId == null)
      {
         if (other.fieldId != null)
            return false;
      }
      else if (!fieldId.equals(other.fieldId))
         return false;
      return true;
   }

}
