/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.servlet;

import java.io.IOException;
import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * A file servlet component for serving up file system resources very that
 * closely resembles that used in DWR but enabled to handle images
 * 
 * @author Rob Monie
 * @author Joe Walker [joe at getahead dot ltd dot uk]
 */
public class FileProcessor
{

   private static Log log = LogFactory.getLog(FileProcessor.class);

   static final String RESOURCES_PATH = "/layoutResources";
   
   /**
    * HTTP etag header
    */
   static final String HEADER_ETAG = "ETag";

   /**
    * HTTP etag equivalent of HEADER_IF_MODIFIED
    */
   static final String HEADER_IF_NONE = "If-None-Match";

   /**
    * HTTP header for when a file was last modified
    */
   static final String HEADER_LAST_MODIFIED = "Last-Modified";

   /**
    * HTTP header to request only modified data
    */
   static final String HEADER_IF_MODIFIED = "If-Modified-Since";

   /**
    * The name of the user agent HTTP header
    */
   static final String HEADER_USER_AGENT = "User-Agent";

   /**
    * MIME constant for plain text. If we need to do more advanced char
    * processing then we should consider adding "; charset=utf-8" to the end of
    * these 3 strings and altering the marshalling to assume utf-8, which it
    * currently does not.
    */
   static final String MIME_PLAIN = "text/plain";

   /**
    * MIME constant for HTML
    */
   static final String MIME_HTML = "text/html";

   /**
    * MIME constant for Javascript
    */
   static final String MIME_JS = "text/javascript";

   /**
    * MIME constant for Javascript
    */
   static final String MIME_CSS = "text/css";

   /**
    * MIME constant for JPG
    */
   static final String MIME_JPG = "image/jpeg";

   /**
    * MIME constant for GIF
    */
   static final String MIME_GIF = "image/gif";

   /**
    * MIME constant for PNG
    */
   static final String MIME_PNG = "image/png";

   /**
    * Basically a file servlet component that does some <b>very limitted</b> EL
    * type processing on the file. See the source for the cheat.
    * 
    * @param req
    *           The request from the browser
    * @param resp
    *           The response channel
    * @param path
    *           The path to search for, process and output
    * @param mimeType
    *           The mime type to use for this output file
    * @throws IOException
    *            If writing to the output fails
    */
   protected void doFile(HttpServletRequest req, HttpServletResponse resp, String path, String mimeType)
         throws IOException
   {
      if (isUpToDate(req, path))
      {
         resp.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
         return;
      }

      InputStream raw = null;

            String resource = RESOURCES_PATH + path;

            raw = getClass().getResourceAsStream(resource);
            
            if (raw == null)
            {
               throw new IOException("Failed to find resource: " + resource); //$NON-NLS-1$
            }


      resp.setContentType(mimeType);
      resp.setDateHeader(HEADER_LAST_MODIFIED, servletContainerStartTime);
      resp.setHeader(HEADER_ETAG, etag);
      
      resp.getOutputStream().write(IOUtils.toByteArray(raw));
      //out.flush();
   }

   /**
    * Do we need to send the conent for this file
    * 
    * @param req
    *           The HTTP request
    * @param path
    *           The file path (for debug purposes)
    * @return true iff the ETags and If-Modified-Since headers say we have not
    *         changed
    */
   private boolean isUpToDate(HttpServletRequest req, String path)
   {
      if (ignoreLastModified)
      {
         return false;
      }

      long modifiedSince = -1;
      try
      {
         // HACK: Webfear appears to get confused sometimes
         modifiedSince = req.getDateHeader(HEADER_IF_MODIFIED);
      }
      catch (RuntimeException ex)
      {
         log
               .warn("Websphere/RAD date failure. If you understand why this might happen please report to dwr-users mailing list"); //$NON-NLS-1$
      }

      if (modifiedSince != -1)
      {
         // Browsers are only accurate to the second
         modifiedSince -= modifiedSince % 1000;
      }
      String givenEtag = req.getHeader(HEADER_IF_NONE);

      // Deal with missing etags
      if (givenEtag == null)
      {
         // There is no ETag, just go with If-Modified-Since
         if (modifiedSince > servletContainerStartTime)
         {
            if (log.isDebugEnabled())
            {
               log
                     .debug("Sending 304 for " + path + " If-Modified-Since=" + modifiedSince + ", Last-Modified=" + servletContainerStartTime); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            }
            return true;
         }

         // There are no modified setttings, carry on
         return false;
      }

      // Deal with missing If-Modified-Since
      if (modifiedSince == -1)
      {
         if (!etag.equals(givenEtag))
         {
            // There is an ETag, but no If-Modified-Since
            if (log.isDebugEnabled())
            {
               log.debug("Sending 304 for " + path + " Old ETag=" + givenEtag + ", New ETag=" + etag); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            }
            return true;
         }

         // There are no modified setttings, carry on
         return false;
      }

      // Do both values indicate that we are in-date?
      if (etag.equals(givenEtag) && modifiedSince <= servletContainerStartTime)
      {
         if (log.isDebugEnabled())
         {
            log.debug("Sending 304 for " + path); //$NON-NLS-1$
         }
         return true;
      }

      return false;
   }

   /**
    * @return Returns the ignoreLastModified.
    */
   public boolean isIgnoreLastModified()
   {
      return ignoreLastModified;
   }

   /**
    * @param ignoreLastModified
    *           The ignoreLastModified to set.
    */
   public void setIgnoreLastModified(boolean ignoreLastModified)
   {
      this.ignoreLastModified = ignoreLastModified;
   }

   /**
    * To what level do we compress scripts?
    * 
    * @param scriptCompressed
    *           The scriptCompressed to set.
    */
   // public void setScriptCompressed(boolean scriptCompressed)
   // {
   // this.scriptCompressed = scriptCompressed;
   // }
   /**
    * @param compressionLevel
    *           The compressionLevel to set.
    */
   // public void setCompressionLevel(int compressionLevel)
   // {
   // this.compressionLevel = compressionLevel;
   // }
   /**
    * The time on the script files
    */
   private static final long servletContainerStartTime;

   /**
    * The etag (=time for us) on the script files
    */
   private static final String etag;

   /**
    * Initialize the container start time
    */
   static
   {
      // Browsers are only accurate to the second
      long now = System.currentTimeMillis();
      servletContainerStartTime = now - (now % 1000);

      etag = "\"" + servletContainerStartTime + '\"'; 
   }

   /**
    * Do we ignore all the Last-Modified/ETags blathering?
    */
   private boolean ignoreLastModified = false;

//   /**
//    * How much do we compression javascript by?
//    */
//   // private int compressionLevel = JavascriptUtil.LEVEL_DEBUGGABLE;
//   /**
//    * Do we retain comments and unneeded spaces in Javascript code?
//    */
//   private boolean scriptCompressed = true;
//
//   /**
//    * We cache the output for speed
//    */
//   private final Map outputCache = new HashMap();

   /**
    * The means by which we strip comments
    */
   // private JavascriptUtil jsutil = new JavascriptUtil();

}
