/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Iterator;

import org.apache.commons.beanutils.PropertyUtils;
import org.springframework.util.StringUtils;

public abstract class AbstractFieldGroupTag extends AbstractHtmlInputAttributesTag
{

   private Collection buttons;
   private String value = null;
   private String label = null;
   protected String prefix = "";
   protected String suffix = "";
   
   
   public AbstractFieldGroupTag()
   {
      super();
   }

   /**
    * Sets the radio button group to be rendered.
    * 
    * @param buttons <code>Collection</code> of JavaBeans, each representing
    *                one radio button. The type of the
    *                <code>Collection</code> elements can be anything,
    *                as long as JavaBean reflection can be performed on them.
    * @see #setValue(String)
    * @see #setLabel(String)
    */
   
   public void setButtonGroup(Collection buttons)
   {
      this.buttons = buttons;
   }
   
   /**
    * Sets the name of the JavaBean property which provides the value
    * of the radio button. This tag uses <code>value</code> to reflectively
    * get the radio button values when iterating the <code>Collection</code>
    * provided to {@link #setButtonGroup(Collection)}.
    * 
    * @param value the name of the JavaBean property which provides
    *                  the value of the radio button
    */
   public void setValue(String value)
   {
      this.value = value;
   }
   
   /**
    * Sets the name of the JavaBean property which provides the label
    * of the radio button. This tag uses <code>label</code> to reflectively
    * get the radio button values when iterating the <code>Collection</code>
    * provided to {@link #setButtonGroup(Collection)}.
    * <p>
    * If this method isn't called, the custom tag doesn't print any label.
    * 
    * @param label the name of the JavaBean property which provides
    *                  the label of the radio button
    */
   public void setLabel(String label)
   {
      this.label = label;
   }
   
   /**
    * Sets the text to be printed before the radio button.
    *  
    * @param prefix the text to be printed before the radio button
    */
   public void setPrefix(String prefix)
   {
      this.prefix = prefix;
   }
   
   /**
    * Sets the text to be printed after the radio button.
    * 
    * @param suffix the text to be printed after the radio button
    */
   public void setSuffix(String suffix)
   {
      this.suffix = suffix;
   }
   
   protected String deriveValue(Object radioObject)
   {
      try
      {
         return PropertyUtils.getProperty(radioObject, this.value).toString();
      }
      catch (IllegalAccessException e)
      {
         throw new RuntimeException(
               "Unable to find property named '" + this.value + "'.", e);
      }
      catch (InvocationTargetException e)
      {
         throw new RuntimeException(
               "Unable to find property named '" + this.value + "'.", e);
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException(
               "Unable to find property named '" + this.value + "'.", e);
      }
   }
   
   protected String deriveLabel(Object radioObject)
   {
      try
      {
         return (this.label == null)?
               "" : PropertyUtils.getProperty(radioObject, this.label).toString();
      }
      catch (IllegalAccessException e)
      {
         throw new RuntimeException(
               "Unable to find property named '" + this.value + "'.", e);
      }
      catch (InvocationTargetException e)
      {
         throw new RuntimeException(
               "Unable to find property named '" + this.value + "'.", e);
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException(
               "Unable to find property named '" + this.value + "'.", e);
      }
   }

   

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#appendEditModeOutput(java.lang.StringBuffer)
    */
   protected void appendEditModeOutput(StringBuffer output)
   {
      int ii = 0;
      for (Iterator i = this.buttons.iterator(); i.hasNext(); )
      {
         Object currentObject = i.next();
         renderEachItem(output, currentObject, ii++);
      }
      String confirmedPath = status == null ? this.pathIfNestedNull : status.getExpression();
      output.append("<input type=\"hidden\" value=\"\" name=\"_"+confirmedPath+"\" id=\"_"+confirmedPath+"\" />");
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#appendReadOnlyOutput(java.lang.StringBuffer)
    */
   protected void appendReadOnlyOutput(StringBuffer output)
   {
      String confirmedValue = status == null ? "" : status.getDisplayValue();
      if(!StringUtils.hasText(confirmedValue) && this.emptyDisplayString != null)
         confirmedValue = this.emptyDisplayString;
      output.append(this.prefix);
      output.append(confirmedValue);
      output.append(this.suffix);
   }

   public abstract void renderEachItem(StringBuffer output, Object item, int index);

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#getGroupSize()
    */
   public int getGroupSize()
   {
      if(this.buttons == null)
      {
         return 0;
      }
      return this.buttons.size();
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#isGroupedField()
    */
   public boolean isGroupedField()
   {
      return true;
   }
   
   

}
