/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspTagException;

import net.sf.springlayout.util.SpringLayoutConfiguration;
import net.sf.springlayout.web.layout.FieldDefinition;
import net.sf.springlayout.web.layout.Form;
import net.sf.springlayout.web.layout.OnPageFieldInstance;
import net.sf.springlayout.web.security.PageAccessHolder;
import net.sf.springlayout.web.security.PageAccessManager;
import net.sf.springlayout.web.util.HtmlUtils;
import net.sf.springlayout.web.util.WebConstants;

import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.util.StringUtils;
import org.springframework.validation.Errors;


/**
 * <p>
 * Abstract class for creating field tags with support for binding to the model 
 * and form field fieldDefinition. Uses the <code>FormTag.CURRENT_PANEL</code>  
 * request attribute to get a handle to the layoutPanel for the current 
 * form being displayed in order to evaluate the current field against it's field 
 * definition.
 * 
 * This tag provides html passthrough for all general javascript events as well as
 * standard htmlAttributes applicable to all labels and fields. This code lends heavily
 * from the Spring form tags.
 * 
 * @author Rob Monie, Adam Boas
 *
 */
public abstract class AbstractFieldTag extends LayoutBindTag
{
   private StringBuffer output;
   protected OnPageFieldInstance fieldInstance;
   protected static SpringLayoutConfiguration config = SpringLayoutConfiguration.getInstance();
   protected String emptyDisplayString = config.getProperty("fieldValue.readonly.empty.default");

   /**
    * Primarily for testing this allows the StringBuffer that this tag and it's children use to
    * append their html output to be set and re-acquired.
    *
    * Retrieves the output StringBuffer which this AbstractFieldTag is using to append html.
    * @return Returns the output which this AbstractFieldTag is using to append html.
    */
   public StringBuffer getOutput()
   {
      return output;
   }

   /**
    * Sets the output which this AbstractFieldTag will use to append html.
    * @param output Sets the output which this AbstractFieldTag will use to append html
    */
   public void setOutput(StringBuffer output)
   {
      this.output = output;
   }

   /**
    * Sets the emptyDisplayString for this InputFieldTag.
    * 
    * @param emptyDisplayString Sets the emptyDisplayString for this InputFieldTag.
    */
   public void setEmptyDisplayString(String emptyDisplayString)
   {
      this.emptyDisplayString = emptyDisplayString;
   }

   /**
    * <p>
    * Sets up a <code>FieldStatus</code> Field based on the current context
    * and config in the field definitions for use by spring field tags.
    * </p>
    * 
    * @see org.springframework.web.servlet.tags.RequestContextAwareTag#doStartTagInternal()
    */
   public int doStartTagInternal() throws Exception
   {
      super.doStartTagInternal();
      //Initialize the outputBuffer for this tag instance
      this.output = new StringBuffer();

      HttpServletRequest request = (HttpServletRequest) this.pageContext.getRequest();
      Form currentForm = (Form) request.getAttribute(WebConstants.CURRENT_FORM);
      if (currentForm == null)
      {
         throw new NullPointerException("Cannot find " + WebConstants.CURRENT_FORM  + 
               " attribute.  Tag must be used in the context of a layout:form tag.");
      }

      PageAccessHolder pageAccessHolder = PageAccessManager.getPageAccessHolder();
      if (pageAccessHolder == null)
      {
         throw new NullPointerException("pageAccessHolder is null.");
      }

      // Get the id for the control which will be rendered
      String controlId;
      FieldDefinition fieldDefinition = getFieldDefinition(request);
      if (this.status != null)
         controlId = status.getExpression();
      else
         controlId = this.pathIfNestedNull;

      fieldInstance = new OnPageFieldInstance(controlId, fieldDefinition,
            pageAccessHolder, this.isGroupedField(), this.getGroupSize());
      addToFieldSet(fieldDefinition, fieldInstance);

      String commandName = request.getAttribute(WebConstants.COMMAND_NAME).toString();

      Errors errors = (Errors) this.getRequestContext().getErrors(commandName);
      if (errors == null)
      {
         throw new NullPointerException("errors is null.");
      }

      if (errors.getFieldError(controlId) != null)
      {
         fieldInstance.setHasErrors(true);
      }

      return doFieldStartTag();

   }

   /**
    * Called in sub classes
    * 
    * @return
    * @throws JspTagException
    */
   public abstract int doFieldStartTag() throws JspTagException;

   /**
    * Builds the ouput which the tag must supply in ReadOnly mode.
    * 
    * @return the ouput which the tag must supply in ReadOnly mode.
    */
   protected void appendReadOnlyOutput(StringBuffer output)
   {
      if (status != null && StringUtils.hasText(status.getDisplayValue()))
         output.append((HtmlUtils.newlineToBr(status.getDisplayValue())));
      else if (this.emptyDisplayString != null)
         output.append(this.emptyDisplayString);
   }

   /**
    * Builds the ouput which the tag must supply in Edit mode.
    * 
    * @return the ouput which the tag must supply in Edit mode.
    */
   protected abstract void appendEditModeOutput(StringBuffer output);

   /**
    * Adds this OnPageFieldInstance to the appropriate set in the
    * fieldDefinition
    * 
    * @param fieldDefinition
    * @param fieldInstance
    */
   abstract protected void addToFieldSet(FieldDefinition fieldDefinition,
         OnPageFieldInstance fieldInstance);

   protected final FieldDefinition getFieldDefinition(HttpServletRequest request)
   {
      Form currentForm = (Form) request.getAttribute(WebConstants.CURRENT_FORM);
      String commandName = request.getAttribute(WebConstants.COMMAND_NAME).toString();
      //Regex to remove the command name and it's index if the command is a List or array
      String fieldDefinitionKey = path.replaceAll(commandName + "(\\[-?\\d+\\])?\\.", "");
      FieldDefinition fieldDefinition = currentForm.getFieldDefinition(fieldDefinitionKey);
      if (fieldDefinition == null)
      {
         throw new NullPointerException("Cannot find fieldDefinition for field with key: " + 
               fieldDefinitionKey);
      }
      return fieldDefinition;
   }

   /**
    * Does this field use a group of options rendered as separate fields such as
    * a radio button or multiple checkboxes with the same name - default is
    * false
    * 
    * @return
    */
   public boolean isGroupedField()
   {
      return false;
   }

   /**
    * Gets the default group size of one for non grouped fields
    * 
    * @return
    */
   public int getGroupSize()
   {
      return 1;
   }

   /**
    * @return the current command wrapped in a BeanWrapper.
    */
   protected BeanWrapper getWrappedCommand()
   {
      String commandName = (String) this.pageContext.getRequest().getAttribute(WebConstants.COMMAND_NAME);
      Object command = this.pageContext.getRequest().getAttribute(commandName);
      return new BeanWrapperImpl(command);
   }
   
}
