/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.jsp.JspTagException;

import org.springframework.util.StringUtils;

/**
 * Class to contain HTML attributes common to all tags.
 * 
 * @author Adam Boas Created on 19/07/2006
 */
public abstract class AbstractHtmlAttributesTag extends AbstractFieldTag
{
   /**
    * The Map of attribute names to values which should be prepended when building the common
    * htmlAttributes.
    */
   protected Map prependMap;

   /**
    * The name of the '<code>class</code>' attribute.
    */
   public static final String CLASS_ATTRIBUTE = "class";

   /**
    * The name of the '<code>style</code>' attribute.
    */
   public static final String STYLE_ATTRIBUTE = "style";

   /**
    * The name of the '<code>lang</code>' attribute.
    */
   public static final String LANG_ATTRIBUTE = "lang";

   /**
    * The name of the '<code>title</code>' attribute.
    */
   public static final String TITLE_ATTRIBUTE = "title";

   /**
    * The name of the '<code>dir</code>' attribute.
    */
   public static final String DIR_ATTRIBUTE = "dir";

   /**
    * The name of the '<code>tabindex</code>' attribute.
    */
   public static final String TABINDEX_ATTRIBUTE = "tabindex";

   /**
    * The name of the '<code>onclick</code>' attribute.
    */
   public static final String ONCLICK_ATTRIBUTE = "onclick";

   /**
    * The name of the '<code>onchange</code>' attribute.
    */
   public static final String ONCHANGE_ATTRIBUTE = "onchange";

   /**
    * The name of the '<code>ondblclick</code>' attribute.
    */
   public static final String ONDBLCLICK_ATTRIBUTE = "ondblclick";

   /**
    * The name of the '<code>onmousedown</code>' attribute.
    */
   public static final String ONMOUSEDOWN_ATTRIBUTE = "onmousedown";

   /**
    * The name of the '<code>onmouseup</code>' attribute.
    */
   public static final String ONMOUSEUP_ATTRIBUTE = "onmouseup";

   /**
    * The name of the '<code>onmouseover</code>' attribute.
    */
   public static final String ONMOUSEOVER_ATTRIBUTE = "onmouseover";

   /**
    * The name of the '<code>onmousemove</code>' attribute.
    */
   public static final String ONMOUSEMOVE_ATTRIBUTE = "onmousemove";

   /**
    * The name of the '<code>onmouseout</code>' attribute.
    */
   public static final String ONMOUSEOUT_ATTRIBUTE = "onmouseout";

   /**
    * The name of the '<code>onkeypress</code>' attribute.
    */
   public static final String ONKEYPRESS_ATTRIBUTE = "onkeypress";

   /**
    * The name of the '<code>onkeyup</code>' attribute.
    */
   public static final String ONKEYUP_ATTRIBUTE = "onkeyup";

   /**
    * The name of the '<code>onkeydown</code>' attribute.
    */
   public static final String ONKEYDOWN_ATTRIBUTE = "onkeydown";

   /**
    * The value of the '<code>class</code>' attribute.
    */
   private String cssClass;

   /**
    * The value of the '<code>class</code>' attribute applied when an there is a validation
    * error.
    */
   protected String cssErrorClass = "error";

   /**
    * The value of the '<code>style</code>' attribute.
    */
   private String cssStyle;

   /**
    * The value of the '<code>lang</code>' attribute.
    */
   private String lang;

   /**
    * The value of the '<code>title</code>' attribute.
    */
   private String title;

   /**
    * The value of the '<code>dir</code>' attribute.
    */
   private String dir;

   /**
    * The value of the '<code>tabindex</code>' attribute.
    */
   private String tabindex;

   /**
    * The value of the '<code>onclick</code>' attribute.
    */
   private String onclick;

   /**
    * The value of the '<code>onchange</code>' attribute.
    */
   private String onchange;

   /**
    * The value of the '<code>ondblclick</code>' attribute.
    */
   private String ondblclick;

   /**
    * The value of the '<code>onmousedown</code>' attribute.
    */
   private String onmousedown;

   /**
    * The value of the '<code>onmouseup</code>' attribute.
    */
   private String onmouseup;

   /**
    * The value of the '<code>onmouseover</code>' attribute.
    */
   private String onmouseover;

   /**
    * The value of the '<code>onmousemove</code>' attribute.
    */
   private String onmousemove;

   /**
    * The value of the '<code>onmouseout</code>' attribute.
    */
   private String onmouseout;

   /**
    * The value of the '<code>onkeypress</code>' attribute.
    */
   private String onkeypress;

   /**
    * The value of the '<code>onkeyup</code>' attribute.
    */
   private String onkeyup;

   /**
    * The value of the '<code>onkeydown</code>' attribute.
    */
   private String onkeydown;

   /**
    * The '<code>mode</code>' which this tag should operated in. If unset will operate in Read
    * or Edit depending on the value in the request parameter. If over-ridden here the tag will
    * always function in the mode specified no matter what mode is requested by the page.
    */
   protected String mode;

   /**
    * No arg constructor which builds a new hashMap for prepending HTML attributes.
    */
   public AbstractHtmlAttributesTag()
   {
      this.prependMap = new HashMap();
   }

   /**
    * Sets the cssClass for this AbstractFieldTag.
    * 
    * @param cssClass Sets the cssClass for this AbstractFieldTag.
    */
   public void setCssClass(String cssClass)
   {
      this.cssClass = cssClass;
   }

   /**
    * Sets the cssErrorClass for this AbstractFieldTag.
    * 
    * @param cssErrorClass Sets the cssErrorClass for this AbstractFieldTag.
    */
   public void setCssErrorClass(String cssErrorClass)
   {
      this.cssErrorClass = cssErrorClass;
   }

   /**
    * Sets the cssStyle for this AbstractFieldTag.
    * 
    * @param cssStyle Sets the cssStyle for this AbstractFieldTag.
    */
   public void setCssStyle(String cssStyle)
   {
      this.cssStyle = cssStyle;
   }

   /**
    * Sets the dir for this AbstractFieldTag.
    * 
    * @param dir Sets the dir for this AbstractFieldTag.
    */
   public void setDir(String dir)
   {
      this.dir = dir;
   }

   /**
    * Sets the lang for this AbstractFieldTag.
    * 
    * @param lang Sets the lang for this AbstractFieldTag.
    */
   public void setLang(String lang)
   {
      this.lang = lang;
   }

   /**
    * Sets the onclick for this AbstractFieldTag.
    * 
    * @param onclick Sets the onclick for this AbstractFieldTag.
    */
   public void setOnclick(String onclick)
   {
      this.onclick = onclick;
   }

   /**
    * Sets the onchange for this AbstractHtmlInputAttributesTag.
    * 
    * @param onchange The onchange for this AbstractHtmlInputAttributesTag.
    */
   public void setOnchange(String onchange)
   {
      this.onchange = onchange;
   }

   /**
    * Sets the ondblclick for this AbstractFieldTag.
    * 
    * @param ondblclick Sets the ondblclick for this AbstractFieldTag.
    */
   public void setOndblclick(String ondblclick)
   {
      this.ondblclick = ondblclick;
   }

   /**
    * Sets the onkeydown for this AbstractFieldTag.
    * 
    * @param onkeydown Sets the onkeydown for this AbstractFieldTag.
    */
   public void setOnkeydown(String onkeydown)
   {
      this.onkeydown = onkeydown;
   }

   /**
    * Sets the onkeypress for this AbstractFieldTag.
    * 
    * @param onkeypress Sets the onkeypress for this AbstractFieldTag.
    */
   public void setOnkeypress(String onkeypress)
   {
      this.onkeypress = onkeypress;
   }

   /**
    * Sets the onkeyup for this AbstractFieldTag.
    * 
    * @param onkeyup Sets the onkeyup for this AbstractFieldTag.
    */
   public void setOnkeyup(String onkeyup)
   {
      this.onkeyup = onkeyup;
   }

   /**
    * Sets the onmousedown for this AbstractFieldTag.
    * 
    * @param onmousedown Sets the onmousedown for this AbstractFieldTag.
    */
   public void setOnmousedown(String onmousedown)
   {
      this.onmousedown = onmousedown;
   }

   /**
    * Sets the onmousemove for this AbstractFieldTag.
    * 
    * @param onmousemove Sets the onmousemove for this AbstractFieldTag.
    */
   public void setOnmousemove(String onmousemove)
   {
      this.onmousemove = onmousemove;
   }

   /**
    * Sets the onmouseout for this AbstractFieldTag.
    * 
    * @param onmouseout Sets the onmouseout for this AbstractFieldTag.
    */
   public void setOnmouseout(String onmouseout)
   {
      this.onmouseout = onmouseout;
   }

   /**
    * Sets the onmouseover for this AbstractFieldTag.
    * 
    * @param onmouseover Sets the onmouseover for this AbstractFieldTag.
    */
   public void setOnmouseover(String onmouseover)
   {
      this.onmouseover = onmouseover;
   }

   /**
    * Sets the onmouseup for this AbstractFieldTag.
    * 
    * @param onmouseup Sets the onmouseup for this AbstractFieldTag.
    */
   public void setOnmouseup(String onmouseup)
   {
      this.onmouseup = onmouseup;
   }

   /**
    * Sets the tabindex for this AbstractFieldTag.
    * 
    * @param tabindex Sets the tabindex for this AbstractFieldTag.
    */
   public void setTabindex(String tabindex)
   {
      this.tabindex = tabindex;
   }

   /**
    * Sets the mode for this AbstractHtmlAttributesTag.
    * 
    * @param mode Sets the mode for this AbstractHtmlAttributesTag.
    */
   public void setMode(String mode)
   {
      this.mode = mode;
   }

   /**
    * Sets the title for this AbstractFieldTag.
    * 
    * @param title Sets the title for this AbstractFieldTag.
    */
   public void setTitle(String title)
   {
      this.title = title;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.taglib.LayoutBindTag#doFieldStartTag()
    */
   public int doFieldStartTag() throws JspTagException
   {
      StringBuffer output = this.getOutput();
      if (fieldInstance.getReadOnly() || "read".equals(this.mode))
      {
         this.appendReadOnlyOutput(output);
      }
      else if (!"multi".equals(this.mode))
      {
         this.appendEditModeOutput(output);
      }
      else
      {
         String confirmedPath = status == null ? this.pathIfNestedNull : status.getExpression();
         output.append("<div class=\"multiContainer\" id=\"");
         output.append(confirmedPath);
         output.append("_container\" onclick=\"swapToEditMode('");
         output.append(confirmedPath);
         output.append("');\"><div id=\"");
         output.append(confirmedPath);
         output.append("_read\">");
         this.appendReadOnlyOutput(output);
         output.append("&nbsp;</div><div id=\"");
         output.append(confirmedPath);
         output.append("_edit\" style=\"display:none;\">");
         this.appendEditModeOutput(output);
         output.append("</div></div>");
         if (this.fieldInstance.getHasErrors())
         {
            output.append("<script type=\"text/javascript\">");
            output.append("triggerEvent('click', $('");
            output.append(confirmedPath);
            output.append("_container'));</script>");
         }
      }
      try
      {
         pageContext.getOut().write(output.toString());
      }
      catch (IOException e)
      {
         throw new JspTagException(e.getMessage());
      }

      return SKIP_BODY;
   }

   /**
    * Appends the general HTML Attributes available to all Tags to the StringBuffer provided.
    * 
    * @param attributeBuffer the StringBuffer to append the HTML Attributes to.
    */
   public void appendCommonHtmlAttributes(StringBuffer attributeBuffer)
   {

      this.appendAttribute(attributeBuffer, TITLE_ATTRIBUTE, this.title);
      this.appendAttribute(attributeBuffer, DIR_ATTRIBUTE, this.dir);
      this.appendAttribute(attributeBuffer, LANG_ATTRIBUTE, this.lang);
      this.appendAttribute(attributeBuffer, STYLE_ATTRIBUTE, this.cssStyle);
      this.appendAttribute(attributeBuffer, TABINDEX_ATTRIBUTE, this.tabindex);
      this.appendAttribute(attributeBuffer, ONCLICK_ATTRIBUTE, this.onclick);
      this.appendAttribute(attributeBuffer, ONCHANGE_ATTRIBUTE, this.onchange);
      this.appendAttribute(attributeBuffer, ONDBLCLICK_ATTRIBUTE, this.ondblclick);
      this.appendAttribute(attributeBuffer, ONKEYDOWN_ATTRIBUTE, this.onkeydown);
      this.appendAttribute(attributeBuffer, ONKEYPRESS_ATTRIBUTE, this.onkeypress);
      this.appendAttribute(attributeBuffer, ONKEYUP_ATTRIBUTE, this.onkeyup);
      this.appendAttribute(attributeBuffer, ONMOUSEDOWN_ATTRIBUTE, this.onmousedown);
      this.appendAttribute(attributeBuffer, ONMOUSEMOVE_ATTRIBUTE, this.onmousemove);
      this.appendAttribute(attributeBuffer, ONMOUSEOUT_ATTRIBUTE, this.onmouseout);
      this.appendAttribute(attributeBuffer, ONMOUSEOVER_ATTRIBUTE, this.onmouseover);
      this.appendAttribute(attributeBuffer, ONMOUSEUP_ATTRIBUTE, this.onmouseup);

   }

   /**
    * Convenience method to append the attribute name value pair to the <code>StringBuffer</code>
    * provided.
    * 
    * @param attributeBuffer StringBuffer to append the name value pair to.
    * @param attributeName name to append
    * @param attributeValue value to append
    */
   protected void appendAttribute(StringBuffer attributeBuffer, String attributeName,
            String attributeValue)
   {
      String prependValue = (String) this.prependMap.get(attributeName);
      if (StringUtils.hasText(attributeValue) || StringUtils.hasText(prependValue))
      {
         attributeBuffer.append(attributeName);
         attributeBuffer.append("=\"");

         if (StringUtils.hasText(prependValue))
            attributeBuffer.append(prependValue);
         if (StringUtils.hasText(attributeValue))
            attributeBuffer.append(attributeValue);
         attributeBuffer.append("\" ");
      }
   }

   protected abstract String getBaseCssClass();

   /**
    * Appends the appropriate CssClasses to the tag, this includes base class for the field type,
    * mandatory and error classes if appropriate
    * 
    * @param cssBuffer stringBuffer to append classes to
    * @param readonly indicates if the field is currently in readonly mode.
    */
   protected void appendAllApplicableCssClasses(StringBuffer cssBuffer, boolean readonly)
   {
      cssBuffer.append("class=\"");
      cssBuffer.append(this.getBaseCssClass());

      if (fieldInstance.isMandatory(this.getWrappedCommand()))
      {
         cssBuffer.append(" mandatory");
      }

      if (fieldInstance.getHasErrors())
      {
         cssBuffer.append(" ");
         cssBuffer.append(this.cssErrorClass);
      }

      if (StringUtils.hasText(this.cssClass))
      {
         cssBuffer.append(" ");
         cssBuffer.append(this.cssClass);
      }

      if (!readonly && this.status == null)
      {
         cssBuffer.append(" disabled");
      }
      cssBuffer.append("\" ");
   }

   /**
    * @return cssClass
    */
   public String getCssClass()
   {
      return cssClass;
   }

   protected boolean isDisabled()
   {
      return status == null;
   }
}
