/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspTagException;

import net.sf.springlayout.web.layout.FieldDefinition;
import net.sf.springlayout.web.layout.OnPageFieldInstance;
import net.sf.springlayout.web.util.WebConstants;

import org.springframework.util.StringUtils;



/**
 * Class to contain HTML attributes comon to all input tags, this includes selects and textareas
 * @author Adam Boas
 * 
 */
public abstract class AbstractHtmlInputAttributesTag extends AbstractHtmlAttributesTag
{
   private static final long serialVersionUID = 439887239188710910L;

   /**
    * The name of the '<code>onfocus</code>' attribute.
    */
   public static final String ONFOCUS_ATTRIBUTE = "onfocus";

   /**
    * The name of the '<code>onblur</code>' attribute.
    */
   public static final String ONBLUR_ATTRIBUTE = "onblur";

   /**
    * The name of the '<code>accesskey</code>' attribute.
    */
   public static final String ACCESSKEY_ATTRIBUTE = "accesskey";

   /**
    * The name of the '<code>disabled</code>' attribute.
    */
   public static final String DISABLED_ATTRIBUTE = "disabled";

   /**
    * The value of the '<code>onfocus</code>' attribute.
    */
   private String onfocus;

   /**
    * The value of the '<code>onblur</code>' attribute.
    */
   private String onblur;

   /**
    * The value of the '<code>accesskey</code>' attribute.
    */
   private String accesskey;

   /**
    * The value of the '<code>disabled</code>' attribute.
    */
   protected String disabled;
   
   /**
    * String to use as context sensitive help for this control.
    */
   private String helpMessage;
   
   /**
    * Internastionalizable message key to use when retrieving the help message for this control.
    */
   private String helpMessageKey;
   
   
   /**
    * @return the help message for this control, either from the message key or the message itself.
    * Returns an empty string if no message set.
    */
   protected String getHelpMessage()
   {
      String message;
      if(this.helpMessage == null && this.helpMessageKey != null)
         message = this.getRequestContext().getMessage(this.helpMessageKey);
      else
         message = this.helpMessage;
      
      return message == null ? "" : message;
   }


   /**
    * Sets the helpMessage for this AbstractHtmlInputAttributesTag.
    * @param helpMessage Sets the helpMessage for this AbstractHtmlInputAttributesTag.
    */
   public void setHelpMessage(String helpMessage)
   {
      this.helpMessage = helpMessage;
   }

   /**
    * Sets the helpMessageKey for this AbstractHtmlInputAttributesTag.
    * @param helpMessageKey Sets the helpMessageKey for this AbstractHtmlInputAttributesTag.
    */
   public void setHelpMessageKey(String helpMessageKey)
   {
      this.helpMessageKey = helpMessageKey;
   }

   /**
    * Sets the accesskey for this AbstractHtmlInputAttributesTag.
    * 
    * @param accesskey
    *           The accesskey for this AbstractHtmlInputAttributesTag.
    */
   public void setAccesskey(String accesskey)
   {
      this.accesskey = accesskey;
   }

   /**
    * Sets the disabled for this AbstractHtmlInputAttributesTag.
    * 
    * @param disabled
    *           The disabled for this AbstractHtmlInputAttributesTag.
    */
   public void setDisabled(String disabled)
   {
      this.disabled = disabled;
   }

   /**
    * Sets the onblur for this AbstractHtmlInputAttributesTag.
    * 
    * @param onblur
    *           The onblur for this AbstractHtmlInputAttributesTag.
    */
   public void setOnblur(String onblur)
   {
      this.onblur = onblur;
   }



   /**
    * Sets the onfocus for this AbstractHtmlInputAttributesTag.
    * 
    * @param onfocus
    *           The onfocus for this AbstractHtmlInputAttributesTag.
    */
   public void setOnfocus(String onfocus)
   {
      this.onfocus = onfocus;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractHtmlAttributesTag#doFieldStartTag()
    */
   public int doFieldStartTag() throws JspTagException
   {
      this.prependMap = new HashMap();
      this.prependMap.put(ONCHANGE_ATTRIBUTE, "markCommandDirty();LAYOUT.validate(this, true, true); ");
      return super.doFieldStartTag();
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.taglib.AbstractHtmlAttributesTag#appendCommonHtmlAttributes(java.lang.StringBuffer)
    */
   public void appendCommonHtmlAttributes(StringBuffer attributeBuffer)
   {
      super.appendCommonHtmlAttributes(attributeBuffer);

      this.appendAttribute(attributeBuffer, ACCESSKEY_ATTRIBUTE, this.accesskey);
      this.appendAttribute(attributeBuffer, DISABLED_ATTRIBUTE, this.disabled);
      this.appendAttribute(attributeBuffer, ONBLUR_ATTRIBUTE, this.onblur);
     
      this.appendAttribute(attributeBuffer, ONFOCUS_ATTRIBUTE, this.onfocus);
   }
   
   
   /**
    * Returns a string containing HTML for rendering a help icon/control.
    * @param outputBuffer buffer to append HTML icon/control to
    * @param confirmedPath path of the control this help is regarding
    */
   protected void appendContextSensitiveHelp(StringBuffer outputBuffer, String confirmedPath)
   {
      String path = ((HttpServletRequest) pageContext.getRequest()).getContextPath();
      
      outputBuffer.append("<img src=\"");
      outputBuffer.append(path);
      outputBuffer.append("/layoutResources/images/icon_help.gif");
      outputBuffer.append("\" alt=\"select for help\" name=\"help_");
      outputBuffer.append(confirmedPath);
      outputBuffer.append("\" id=\"help_");
      outputBuffer.append(confirmedPath);
      outputBuffer.append("\" align=\"top\" onclick=\"showHelp(this, '");
      outputBuffer.append(this.getHelpMessage());
      outputBuffer.append("');\" ");
      outputBuffer.append("onmouseover=\"over(this, 'helpIcon');\" ");
      outputBuffer.append("onmouseout=\"out(this,'helpIcon');\"/>");

   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractHtmlAttributesTag#addToFieldSet(net.sf.springlayout.web.layout.FieldDefinition, net.sf.springlayout.web.layout.OnPageFieldInstance)
    */
   protected void addToFieldSet(FieldDefinition fieldDefinition, OnPageFieldInstance fieldInstance)
   {
     AbstractFormTag formTag = (AbstractFormTag)  this.pageContext.getAttribute(WebConstants.LAYOUT_FORM_TAG);
     List onPageFieldInstances = formTag.getOnPageFieldInstances(); 
     onPageFieldInstances.add(fieldInstance);      
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractHtmlAttributesTag#isDisabled()
    */
   protected boolean isDisabled()
   {
       return super.isDisabled() && StringUtils.hasText(this.disabled);
   }
   
   
}
