/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.lang.reflect.InvocationTargetException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;

import net.sf.springlayout.web.layout.FieldDefinition;

import org.apache.commons.beanutils.BeanUtils;
import org.springframework.util.StringUtils;

/**
 * Generates an input field bound to the specified path and field definition.
 * Allows html attributes to be passed throught to the input control as well as
 * client-side type validation.
 * 
 * @author Rob Monie, Adam Boas
 * 
 */
public class AutoCompleteFieldTag extends AbstractHtmlInputAttributesTag
{
   private static final long serialVersionUID = 6770277140854014151L;

   private static String TYPE_JS = "javascript";
   private static String TYPE_URL = "url";

   private String label;
   private String value;
   private String optionsTarget;
   private String optionsTargetType;

   /**
    * @param label
    */
   public void setLabel(String label)
   {
      this.label = label;
   }

   /**
    * @param value
    */
   public void setValue(String value)
   {
      this.value = value;
   }

   /**
    * @param optionsTarget
    */
   public void setOptionsTarget(String optionsTarget)
   {
      this.optionsTarget = optionsTarget;
   }

   /**
    * @param optionsTargetType
    */
   public void setOptionsTargetType(String optionsTargetType)
   {
      this.optionsTargetType = optionsTargetType;
   }

   /**
    * The name of the '<code>maxlength</code>' attribute.
    */
   public static final String MAXLENGTH_ATTRIBUTE = "maxlength";

   /**
    * The name of the '<code>alt</code>' attribute.
    */
   public static final String ALT_ATTRIBUTE = "alt";

   /**
    * The name of the '<code>onselect</code>' attribute.
    */
   public static final String ONSELECT_ATTRIBUTE = "onselect";

   /**
    * The name of the '<code>size</code>' attribute.
    */
   public static final String SIZE_ATTRIBUTE = "size";

   /**
    * The name of the '<code>readonly</code>' attribute.
    */
   public static final String READONLY_ATTRIBUTE = "readonly";

   /**
    * The value of the '<code>maxlength</code>' attribute.
    */
   private String maxlength;

   /**
    * The value of the '<code>alt</code>' attribute.
    */
   private String alt;

   /**
    * The value of the '<code>onselect</code>' attribute.
    */
   private String onselect;

   /**
    * The value of the '<code>type</code>' attribute.
    */
   private String type = "text";

   /**
    * The value of the '<code>size</code>' attribute.
    */
   private String size;

   /**
    * The value of the '<code>readonly</code>' attribute.
    */
   private String readonly;

   /**
    * The value of the Javascript type validation to apply to this field.
    */
   private String javascriptTypeValidation;

   /**
    * Sets the alt for this InputFieldTag.
    * 
    * @param alt
    *           Sets the alt for this InputFieldTag.
    */
   public void setAlt(String alt)
   {
      this.alt = alt;
   }

   /**
    * Sets the maxlength for this InputFieldTag.
    * 
    * @param maxlength
    *           Sets the maxlength for this InputFieldTag.
    */
   public void setMaxlength(String maxlength)
   {
      this.maxlength = maxlength;
   }

   /**
    * Sets the onselect for this InputFieldTag.
    * 
    * @param onselect
    *           Sets the onselect for this InputFieldTag.
    */
   public void setOnselect(String onselect)
   {
      this.onselect = onselect;
   }

   /**
    * Sets the type for this InputFieldTag.
    * 
    * @param type
    *           The type for this InputFieldTag.
    */
   public void setType(String type)
   {
      this.type = type;
   }

   /**
    * Sets the readonly for this InputFieldTag.
    * 
    * @param readonly
    *           Sets the readonly for this InputFieldTag.
    */
   public void setReadonly(String readonly)
   {
      this.readonly = readonly;
   }

   /**
    * Sets the size for this InputFieldTag.
    * 
    * @param size
    *           Sets the size for this InputFieldTag.
    */
   public void setSize(String size)
   {
      this.size = size;
   }

   /**
    * Attaches JavaScript validation to the rendered field. For flexibility,
    * this method takes a value which looks like JavaScript function call. For
    * example, the following value
    * 
    * <pre>
    * decimal(2);
    * range(0, 999);
    * </pre>
    * 
    * would cause the following function calls to be rendered
    * 
    * <pre>
    *  validateDecimal(&lt;field-name&gt;, 2, true) &amp;&amp; validateRange(&lt;field-name&gt;, 0, 999, true)
    * </pre>
    */
   public void setJavascriptTypeValidation(String javascriptTypeValidation)
   {
      this.javascriptTypeValidation = javascriptTypeValidation;
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#buildEditModeOutput()
    */
   protected void appendEditModeOutput(StringBuffer output)
   {
      String confirmedPath = status == null ? this.pathIfNestedNull : status
            .getExpression();
      String confirmedValue = status == null ? "" : status.getDisplayValue();

      Object valueObj = null;
      Object labelObj = null;
      try
      {
         
         valueObj = status.getValue() == null ? null : BeanUtils.getProperty(status.getValue(), value);
         labelObj = status.getValue() == null ? null : BeanUtils.getProperty(status.getValue(), label);
      }
      catch (IllegalAccessException e)
      {
         e.printStackTrace();
      }
      catch (InvocationTargetException e)
      {

         e.printStackTrace();
      }
      catch (NoSuchMethodException e)
      {
         e.printStackTrace();
      }

      String actualValue = valueObj == null ? "" : valueObj.toString();
      String actualLabel = labelObj == null ? "" : labelObj.toString();

      // render the display field
      output.append("<input type=\"");
      output.append("text");
      output.append("\" name=\"");
      output.append(confirmedPath);
      output.append("_display");
      output.append("\" id=\"");
      output.append(confirmedPath);
      output.append("_display");
      output.append("\" value=\"");
      output.append(actualLabel);
      output.append("\" ");
      if (status == null)
         output.append("disabled=\"disabled\" ");
      this.appendCommonHtmlAttributes(output);
      this.appendInputAttributes(output);
      if (!"hidden".equals(type))
         this.appendAllApplicableCssClasses(output, false);
      output.append("/>");

      // render the hidden 'value' field
      output.append("<input type=\"");
      output.append("hidden");
      output.append("\" name=\"");
      output.append(confirmedPath);
      output.append("\" id=\"");
      output.append(confirmedPath);
      output.append("\" value=\"");
      output.append(actualValue);
      output.append("\" ");
      if (status == null)
         output.append("disabled=\"disabled\" ");
      this.appendCommonHtmlAttributes(output);
      this.appendInputAttributes(output);

      output.append("/>");

      output.append("<div id=\"");
      output.append(confirmedPath);
      output.append("_choices\"");
      output.append(" class=\"autocomplete\"/>");

//      if (optionsTargetType.equals(TYPE_JS))
//      {
//         output.append("<script type=\"text/javascript\">");
//         output.append("new Autocompleter.JsFunction(\"");
//         output.append(confirmedPath);
//         output.append("\", \"");
//         output.append(confirmedPath);
//         output.append("_choices");
//         output.append("\", \"");
//         output.append(optionsTarget);
//         output.append("\", {});");
//         output.append("</script>");
//
//      }
//      else if (optionsTargetType.equals(TYPE_URL))
//      {
//         output.append("<script type=\"text/javascript\">");
//         output.append("new Ajax.Autocompleter(\"");
//         output.append(confirmedPath);
//         output.append("\", \"");
//         output.append(confirmedPath);
//         output.append("_choices");
//         output.append("\", \"");
//         output.append(optionsTarget);
//         output.append("\", {});");
//         output.append("</script>");
//      }
      

   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#buildReadOnlyOutput()
    */
   protected void appendReadOnlyOutput(StringBuffer output)
   {

      super.appendReadOnlyOutput(output);

   }

   /**
    * Appends the HTML Attributes specific to an input tag to the StringBuffer
    * provided.
    * 
    * @param attributeBuffer
    *           the StringBuffer to append the HTML Attributes to.
    */
   protected void appendInputAttributes(StringBuffer attributeBuffer)
   {
      if (StringUtils.hasText(this.alt))
         this.appendAttribute(attributeBuffer, ALT_ATTRIBUTE, this.alt);

      String confirmedMaxLength = this.getMaxLength();
      if (StringUtils.hasText(confirmedMaxLength))
         this.appendAttribute(attributeBuffer, MAXLENGTH_ATTRIBUTE,
               confirmedMaxLength);

      String confirmedSize = this.getSize(confirmedMaxLength);
      if (StringUtils.hasText(confirmedSize))
         this.appendAttribute(attributeBuffer, SIZE_ATTRIBUTE, confirmedSize);

      if (StringUtils.hasText(this.readonly))
         this.appendAttribute(attributeBuffer, READONLY_ATTRIBUTE,
               this.readonly);
      if (StringUtils.hasText(this.onselect))
         this.appendAttribute(attributeBuffer, ONSELECT_ATTRIBUTE,
               this.onselect);
   }

   /**
    * Convenience method to retrieve the correct MAxLength for this tag. Allows
    * the MaxLength to be specified in the fieldDefinition or to be set or
    * over-ridden by the tag.
    * 
    * @return the maxLength for this Tag.
    */
   private String getMaxLength()
   {
      if (StringUtils.hasText(this.maxlength))
         return this.maxlength;

      HttpServletRequest request = (HttpServletRequest) this.pageContext
            .getRequest();
      FieldDefinition fieldDefinition = getFieldDefinition(request);
      int definitionMaxLength = fieldDefinition.getMaxLength();
      if (definitionMaxLength != -1)
         return Integer.toString(definitionMaxLength);

      return null;
   }

   /**
    * Convenience method to retrieve the correct size for this tag. Allows the
    * size to be set as a result of the MaxLength or to be over-ridden by the
    * tag.
    * 
    * @param confirmedMaxLength
    * @return the size appropriate for this tag.
    */
   private String getSize(String confirmedMaxLength)
   {
      if (StringUtils.hasText(this.size))
         return this.size;
      if (StringUtils.hasText(confirmedMaxLength))
         return confirmedMaxLength;

      return null;
   }


   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#getBaseCssClass()
    */
   protected String getBaseCssClass()
   {
      return "inputField autocomplete";
   }

   private static final class FunctionCall
   {
      private String name;
      private StringBuffer args = new StringBuffer();

      private FunctionCall(String name, String args)
      {
         this.name = "validate" + capitalizeFirstChar(name.trim());
         this.args.append(args);
      }

      private static FunctionCall[] parseFromCode(String code)
      {
         String[] callsInString = code.split(";");
         FunctionCall[] calls = new FunctionCall[callsInString.length];

         for (int i = 0; i < callsInString.length; i++)
         {
            Matcher matcher = Pattern.compile("(\\w*)\\((.*)\\)").matcher(
                  callsInString[i].trim());
            matcher.matches();

            String funcName = matcher.group(1);
            String argList = (matcher.groupCount() == 1) ? "" : matcher
                  .group(2);
            calls[i] = new FunctionCall(funcName, argList);
         }

         return calls;
      }

      private void addFirstArgument(String arg)
      {
         boolean alreadyHasArg = (this.args.length() > 0);
         if (alreadyHasArg)
         {
            arg = arg + ',';
         }
         this.args.insert(0, arg);
      }

      private void addLastArgument(String arg)
      {
         boolean alreadyHasArg = (this.args.length() > 0);
         if (alreadyHasArg)
         {
            this.args.append(',');
         }
         this.args.append(arg);
      }

      private String capitalizeFirstChar(String orig)
      {
         if (orig.length() == 1)
         {
            return orig.toUpperCase();
         }

         if (orig.length() > 1)
         {
            String firstChar = orig.substring(0, 1);
            String remainingChars = orig.substring(1);
            return firstChar.toUpperCase() + remainingChars;
         }

         return orig;
      }

      public String toString()
      {
         return this.name + "(" + this.args + ")";
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.layout.taglib.AbstractHtmlAttributesTag#isDisabled()
    */
   protected boolean isDisabled()
   {
      return super.isDisabled() && !type.equals("hidden");
   }

}
