/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.IterationTag;
import javax.servlet.jsp.tagext.Tag;

import net.sf.springlayout.web.layout.LayoutForm;
import net.sf.springlayout.web.security.PageAccessHolder;
import net.sf.springlayout.web.security.PageAccessManager;
import net.sf.springlayout.web.util.WebConstants;

import org.springframework.beans.BeanWrapper;
import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.support.RequestContext;
import org.springframework.web.servlet.tags.RequestContextAwareTag;


/**
 * @auther Adam Boas
 * Created on Jan 3, 2007
 */
public class DataGridTag extends BodyTagSupport
{
   private static final long serialVersionUID = -5735925709925672588L;
   
   public static final String GRID_INDEX = "gridIndex";
   public static final String CURRENT_COLLECTION_ITEM = "currentItem";
   public static final String CURRENT_ROW_PATH = "row";
   private static final String DEFAULT_EMPTY_MESSAGE = "No items to display";
   
   //Private variables used internally by the grid
   private Iterator iterator;
   private Object current;
   private boolean isEvaluatingHead;
   private boolean isEvaluatingEmptyContent;
   private int index;
   private int columnCount;
   private RequestContext requestContext;
   private Object items;
   
   //Fields settable by the user.
   /**
    * ID given the the HTML table this grid generates.
    */
   private String gridId;
   /**
    * Path to the Collection in the command object which this grid will iterate through. Can be any 
    * type of Collection, Map or Array but must be contained in the current command object.
    */
   private String path;
   /**
    * HTML passthrough equivelent to 'style' applied to the table this grid generates.
    */
   private String cssStyle;
   /**
    * CSS Class applierd to the HTML table this grid generates.
    */
   private String cssClass;
   /**
    * CSS Class applied to the 'tr' this grid generates for the header row.
    */
   private String headerClass;
   /**
    * CSS Class applied to the 'tr' for the final, footer row
    */
   private String footerRowClass;
   /**
    * CSS Class applied to the 'td' for the final, footer cell. this cell will have a colspan equal to
    * the number of columns in the table.
    */
   private String footerCellClass;
   /**
    * The text which will appear on the 'add' button.
    */
   private String addButtonTitle;
   /**
    * Internastionalizable message key to use when retrieving the title for the 'add' button. Will
    * only be used if an addButtonTitle is not present.
    */
   private String addButtonTitleKey;
   /**
    * CSS Class which will be applied to the 'add' button for this grid.
    */
   private String addButtonClass;
   /**
    * The text which will appear on the 'add' button.
    */
   private String cancelButtonTitle;
   /**
    * Internastionalizable message key to use when retrieving the title for the 'cancel' button. Will
    * only be used if an cancelButtonTitle is not present.
    */
   private String cancelButtonTitleKey;
   /**
    * CSS Class which will be applied to the 'cancel' button for this grid.
    */
   private String cancelButtonClass;
   /**
    * key used to retrieve the message to display if the list to iterate through is empty.
    */
   private String emptyMessageKey;
   /**
    * Enforce that select controls identified by this comma separated list of paths maintain 
    * uniqueness across the grid.
    */
   private String maintainUniqueSelectPaths;
   
   /**
    * Sets the maintainUniqueSelectPaths for this DataGridTag.
    * @param maintainUniqueSelectPaths Sets the maintainUniqueSelectPaths for this DataGridTag.
    */
   public void setMaintainUniqueSelectPaths(String maintainUniqueSelectPaths)
   {
      StringBuffer paths = new StringBuffer();
      String[] pathArray = maintainUniqueSelectPaths.split(",");
      for(int ii = 0; ii < pathArray.length; ii++)
      {
         if(paths.length() > 0)
            paths.append(",");
         paths.append(this.getCorrectedPath(pathArray[ii]));
      }
      this.maintainUniqueSelectPaths = paths.toString();
   }

   /* (non-Javadoc)
    * @see javax.servlet.jsp.tagext.BodyTagSupport#doStartTag()
    */
   public int doStartTag() throws JspException
   {
	   
	   HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
      //reset any variable which my have been used previously
      BeanWrapper wrapper = (BeanWrapper)this.pageContext.getAttribute(WebConstants.BEAN_WRAPPER);
      String correctedPath = this.getCorrectedPath(this.path);
      if(!StringUtils.hasText(correctedPath))
         this.items = wrapper.getWrappedInstance();
      else
         this.items = wrapper.getPropertyValue(correctedPath);
      this.isEvaluatingHead = true;
      this.isEvaluatingEmptyContent = false;
      this.index = -1;
      columnCount = 0;
      this.iterator = this.getIterator();
      this.requestContext = (RequestContext) this.pageContext.getAttribute(
            RequestContextAwareTag.REQUEST_CONTEXT_PAGE_ATTRIBUTE);
      
      String addListVals = (String) request.getAttribute("dataGrid_addList_" + this.gridId);
      String removeListVals = (String) request.getAttribute("dataGrid_removeList_" + this.gridId);
      addListVals = addListVals == null ? "" : addListVals;
      removeListVals = removeListVals == null ? "" : removeListVals;
      
      StringBuffer tableOutput = new StringBuffer();
      tableOutput.append("<input type=\"hidden\" name=\"dataGrid_addList_");
      tableOutput.append(this.gridId);
      tableOutput.append("\" value=\"");
      tableOutput.append(addListVals);
      tableOutput.append("\"/>\n");
      tableOutput.append("<input type=\"hidden\" name=\"dataGrid_removeList_");
      tableOutput.append(this.gridId);
      tableOutput.append("\" id=\"dataGrid_removeList_");
      tableOutput.append(this.gridId);
      tableOutput.append("\" value=\"");
      tableOutput.append(removeListVals);
      tableOutput.append("\"/>\n");
      tableOutput.append("<input type=\"hidden\" name=\"dataGridIds");
      tableOutput.append("\" value=\"");
      tableOutput.append(this.gridId);
      tableOutput.append("\"/>\n");
      tableOutput.append("<table");
      tableOutput.append(this.getIdString());
      tableOutput.append(this.getCssClass());
      tableOutput.append(this.getCssStyle());
      tableOutput.append(">\n");
      write(this.pageContext.getOut(), tableOutput.toString());
      this.doStartRow();
      return Tag.EVAL_BODY_INCLUDE;
   }

   /* (non-Javadoc)
    * @see javax.servlet.jsp.tagext.BodyTagSupport#doAfterBody()
    */
   public int doAfterBody() throws JspException
   {
      this.doEndRow();
      if(this.index == -1)
      {
         this.doStartRow();
         return IterationTag.EVAL_BODY_AGAIN;
      }
      
      if(this.iterator.hasNext())
      {
         this.doStartRow();
         return IterationTag.EVAL_BODY_AGAIN;
      }
      else if(this.getItemsSize() == 0 && !this.isEvaluatingEmptyContent)
      {
         this.isEvaluatingEmptyContent = true;
         this.doStartRow();
         StringBuffer output = new StringBuffer();
         output.append("<td colspan=\"");
         output.append(this.columnCount);
         output.append("\">");
         String emptyMessage = this.emptyMessageKey == null ? DEFAULT_EMPTY_MESSAGE : 
            this.requestContext.getMessage(this.emptyMessageKey);
         output.append(emptyMessage);
         output.append("</td></tr>");
         write(this.pageContext.getOut(), output.toString());
         return IterationTag.EVAL_BODY_AGAIN;
      }
      else
         return Tag.SKIP_BODY;
   }

   /* (non-Javadoc)
    * @see javax.servlet.jsp.tagext.BodyTagSupport#doEndTag()
    */
   public int doEndTag() throws JspException
   {
      LayoutForm form = 
         (LayoutForm)this.pageContext.getRequest().getAttribute(WebConstants.CURRENT_FORM);
      StringBuffer output = new StringBuffer();
      output.append(this.getFooter());
      output.append("</table>\n<script type=\"text/javascript\">");
      
      output.append("var ");
      output.append(this.gridId);
      output.append("_table = ");
      output.append("$('");
      output.append(this.gridId);
      output.append("');");
      
      output.append(this.gridId);
      output.append("_table.nextIndex = ");
      output.append(this.index == -1 ? 0 :this.index );
      output.append(";");
      
      output.append(this.gridId);
      output.append("_table.path = '");
      output.append(this.getCorrectedPath(this.path));
      output.append("';");
      
      output.append(this.gridId);
      output.append("_table.formName = '");
      output.append(form.getFormName());
      output.append("';");
      if(PageAccessManager.getPageAccessHolder().isEditMode())
      {
         if(StringUtils.hasText(this.maintainUniqueSelectPaths))
         {
            output.append("addEvent(window, 'load', function(){loadUniqueSelectOptions('");
            output.append(this.gridId);
            output.append("', '");
            output.append(this.maintainUniqueSelectPaths);
            output.append("');});");
         }
      }
      output.append("</script>");
      write(this.pageContext.getOut(), output.toString());
      this.pageContext.removeAttribute("gridIndex");
      return Tag.EVAL_PAGE;
   }
   
   /**
    * Helper method to write the beginning of each row in the table and expose the necessary 
    * variables for child tags.
    * @throws JspException
    */
   private void doStartRow() throws JspException
   {
      StringBuffer rowOutput = new StringBuffer();
      if(this.index == -1 && !this.isEvaluatingHead)
      {
         rowOutput.append("<tr id=\"");
         rowOutput.append(this.gridId);
         rowOutput.append("_extra\" style=\"visibility:collapse;display:none;\"");
         rowOutput.append(this.getRowClass());
         rowOutput.append(">");
      }
      else if(this.isEvaluatingEmptyContent)
      {
         rowOutput.append("<tr id=\"");
         rowOutput.append(this.gridId);
         rowOutput.append("_empty\"");
         rowOutput.append(this.getRowClass());
         rowOutput.append(">");
      }
      else
      {
         rowOutput.append("<tr");
         if(!this.isEvaluatingHead)
         {
            rowOutput.append(" id=\"");
            rowOutput.append(this.gridId);
            rowOutput.append("_");
            rowOutput.append(this.index);
            rowOutput.append("\"");
            rowOutput.append(this.getRowClass());
         }
         else
            rowOutput.append(this.getHeaderClass());
         rowOutput.append(">");
      }
      DataGridTag.write(this.pageContext.getOut(), rowOutput.toString());
      
      this.pageContext.setAttribute(DataGridTag.GRID_INDEX, new Integer(this.index));
      this.pageContext.setAttribute(DataGridTag.CURRENT_COLLECTION_ITEM, current);
      this.pageContext.setAttribute(DataGridTag.CURRENT_ROW_PATH, this.path + "[" + this.index + "]");
      if(!this.isEvaluatingHead && this.index >= 0 && !this.isEvaluatingEmptyContent)
         this.current = this.iterator.next();
   }
   
   /**
    * Helper method to write the end of each row in the table and remove/replace the necessary 
    * variables inserted for child tags.
    * @throws JspException
    */
   private void doEndRow() throws JspException
   {
      write(this.pageContext.getOut(), "</tr>");
      if(! this.isEvaluatingHead && !this.isEvaluatingEmptyContent)
         this.index++;
      this.isEvaluatingHead = false;
   }
   
   
   /**
    * @return footer row for the grid.
    */
   private String getFooter()
   {
      PageAccessHolder pageAccessHolder = PageAccessManager.getPageAccessHolder();
      StringBuffer footerOutput = new StringBuffer();
      footerOutput.append("<tr");
      footerOutput.append(this.getFooterRowClass());
      footerOutput.append("><td colspan=\"");
      footerOutput.append(this.columnCount);
      footerOutput.append("\"");
      footerOutput.append(this.getFooterCellClass());
      footerOutput.append(">");
      if(pageAccessHolder.isEditMode())
      {
         this.appendCancelButton(footerOutput);
         this.appendAddButton(footerOutput);
      }
      footerOutput.append("</td></tr>");
      
      return footerOutput.toString();
   }
   
   
   /**
    * Appends the addButton to the provided StringBuffer complete with class and label text.
    * @param output String Buffer to append the button to.
    */
   private void appendAddButton(StringBuffer output)
   {
      output.append("<button id=\"");
      output.append(this.gridId);
      output.append("_addButton\"");
      output.append(this.getAddButtonClass());
      output.append("type=\"button\" onclick=\"addRow('");
      output.append(this.gridId);
      output.append("');\">");
      output.append(this.getAddButtonTitleText());
      output.append("</button>");
   }

   /**
    * Appends the cancelButton to the provided StringBuffer complete with class and label text.
    * @param output String Buffer to append the button to.
    */
   private void appendCancelButton(StringBuffer output)
   {
      output.append("<button id=\"");
      output.append(this.gridId);
      output.append("_cancelButton\"");
      output.append(this.getCancelButtonClass());
      output.append("type=\"button\" onclick=\"cancelRow('");
      output.append(this.gridId);
      output.append("');\" style=\"position:absoute;visibility:hidden;\">");
      output.append(this.getCancelButtonTitleText());
      output.append("</button>");
   }
   
   /**
    * @return an Iterator for the items object for this DataGrid.
    */
   private Iterator getIterator()
   {
      if(items instanceof Collection)
         return ((Collection)items).iterator();
      if(this.items.getClass().isArray())
         return Arrays.asList((Object[])this.items).iterator();
      if(this.items instanceof Map)
         return ((Map)this.items).values().iterator();
      
      return null;
   }
   
   
   /**
    * @return size fo the items object which this grid is iterating over.
    */
   private int getItemsSize()
   {
      if(items instanceof Collection)
         return ((Collection)items).size();
      if(this.items.getClass().isArray())
         return ((Object[])this.items).length;
      if(this.items instanceof Map)
         return ((Map)this.items).values().size();
      
      return -1;
   }
   
   /**
    * Retrieves the items for this DataGridTag.
    * @return Returns the items for this DataGridTag.
    */
   public Object getItems()
   {
      return items;
   }

   /**
    * Sets the gridId for this DataGridTag.
    * @param id Sets the gridId for this DataGridTag.
    */
   public void setGridId(String gridId)
   {
      this.gridId = gridId;
   }
   
   /**
    * Retrieves the gridId for this DataGridTag.
    * @return Returns the gridId for this DataGridTag.
    */
   String getGridId()
   {
      return this.gridId;
   }
   
   
   /**
    * @return the lbel and value for the id attribute or an empty string if none has been set.
    */
   private String getIdString()
   {
      if(StringUtils.hasText(this.gridId))
         return " id=\"" + this.gridId + "\"";
      return "";
   }

   
   /**
    * Retrieves the cssClass for this DataGridTag.
    * @return Returns the cssClass for this DataGridTag.
    */
   private String getCssClass()
   {
      if(StringUtils.hasText(this.cssClass))
         return " class=\"" + this.cssClass + "\"";
      return "";
   }

   /**
    * Sets the cssClass for this DataGridTag.
    * @param cssClass Sets the cssClass for this DataGridTag.
    */
   public void setCssClass(String cssClass)
   {
      this.cssClass = cssClass;
   }

   /**
    * Retrieves the headerClass for this DataGridTag.
    * @return Returns the headerClass for this DataGridTag.
    */
   public String getHeaderClass()
   {
      if(StringUtils.hasText(this.headerClass))
         return " class=\"" + this.headerClass + "\"";
      return "";
   }

   /**
    * Sets the headerClass for this DataGridTag.
    * @param headerClass Sets the headerClass for this DataGridTag.
    */
   public void setHeaderClass(String headerClass)
   {
      this.headerClass = headerClass;
   }

   /**
    * Retrieves the cssStyle for this DataGridTag.
    * @return Returns the cssStyle for this DataGridTag.
    */
   public String getCssStyle()
   {
      if(StringUtils.hasText(this.cssStyle))
         return " style=\"" + this.cssStyle + "\"";
      return "";
   }

   /**
    * Sets the cssStyle for this DataGridTag.
    * @param cssStyle Sets the cssStyle for this DataGridTag.
    */
   public void setCssStyle(String cssStyle)
   {
      this.cssStyle = cssStyle;
   }

   /**
    *@return the label and value for the class to be applied to the add button of this grid or an 
    *empty string if none has been set.
    */
   private String getAddButtonClass()
   {
      if(StringUtils.hasText(this.addButtonClass))
         return " class=\"" + this.addButtonClass + "\"";
      return "";
   }

   /**
    * Sets the addButtonClass for this DataGridTag.
    * @param addButtonClass Sets the addButtonClass for this DataGridTag.
    */
   public void setAddButtonClass(String addButtonClass)
   {
      this.addButtonClass = addButtonClass;
   }

   /**
    * Sets the addButtonTitle for this DataGridTag.
    * @param addButtonTitle Sets the addButtonTitle for this DataGridTag.
    */
   public void setAddButtonTitle(String addButtonTitle)
   {
      this.addButtonTitle = addButtonTitle;
   }
   
   /**
    * @return the correct title for this column (either text or a message value).
    */
   private String getAddButtonTitleText() 
   {
      String titleText;
      if(this.addButtonTitle == null && this.addButtonTitleKey != null)
         titleText = this.requestContext.getMessage(this.addButtonTitleKey);
      else
         titleText = this.addButtonTitle;
      
      return titleText == null ? "" :titleText;
   }

   /**
    * Sets the addButtonTitleKey for this DataGridTag.
    * @param addButtonTitleKey Sets the addButtonTitleKey for this DataGridTag.
    */
   public void setAddButtonTitleKey(String addButtonTitleKey)
   {
      this.addButtonTitleKey = addButtonTitleKey;
   }

   /**
    * @return the label and value for the class to be applied to the cancel button of this grid or an 
    * empty string if none has been set.
    */
   private String getCancelButtonClass()
   {
      if(StringUtils.hasText(this.cancelButtonClass))
         return " class=\"" + this.cancelButtonClass + "\"";
      return "";
   }

   /**
    * Sets the cancelButtonClass for this DataGridTag.
    * @param cancelButtonClass Sets the cancelButtonClass for this DataGridTag.
    */
   public void setCancelButtonClass(String cancelButtonClass)
   {
      this.cancelButtonClass = cancelButtonClass;
   }

   /**
    * Sets the cancelButtonTitle for this DataGridTag.
    * @param cancelButtonTitle Sets the cancelButtonTitle for this DataGridTag.
    */
   public void setCancelButtonTitle(String cancelButtonTitle)
   {
      this.cancelButtonTitle = cancelButtonTitle;
   }

   /**
    * @return the correct title for this grid's cancel button (either text or a message value).
    */
   private String getCancelButtonTitleText() 
   {
      String titleText;
      if(this.cancelButtonTitle == null && this.cancelButtonTitleKey != null)
         titleText = this.requestContext.getMessage(this.cancelButtonTitleKey);
      else
         titleText = this.cancelButtonTitle;
      
      return titleText == null ? "" :titleText;
   }

   /**
    * Sets the cancelButtonTitleKey for this DataGridTag.
    * @param cancelButtonTitleKey Sets the cancelButtonTitleKey for this DataGridTag.
    */
   public void setCancelButtonTitleKey(String cancelButtonTitleKey)
   {
      this.cancelButtonTitleKey = cancelButtonTitleKey;
   }

   /**
    * @return the label and value for the class to be applied to the footer td of this grid or an 
    * empty string if none has been set.
    */
   private String getFooterCellClass()
   {
      if(StringUtils.hasText(this.footerCellClass))
         return " class=\"" + this.footerCellClass + "\"";
      return "";
   }

   /**
    * Sets the footerCellClass for this DataGridTag.
    * @param footerCellClass Sets the footerCellClass for this DataGridTag.
    */
   public void setFooterCellClass(String footerCellClass)
   {
      this.footerCellClass = footerCellClass;
   }

   /**
    * @return the label and value for the class to be applied to the footer tr of this grid or an 
    * empty string if none has been set.
    */
   private String getFooterRowClass()
   {
      if(StringUtils.hasText(this.footerRowClass))
         return " class=\"" + this.footerRowClass + "\"";
      return "";
   }

   /**
    * Sets the footerRowClass for this DataGridTag.
    * @param footerRowClass Sets the footerRowClass for this DataGridTag.
    */
   public void setFooterRowClass(String footerRowClass)
   {
      this.footerRowClass = footerRowClass;
   }

   /**
    * @param out
    * @param output
    * @throws JspException
    */
   public static void write(JspWriter out, String output) throws JspException
   {
      try
      {
         out.println(output);
      }
      catch(IOException e)
      {
         throw new JspException(e);
      }
   }
   
   /**
    * @return true or false to indicate if this tag is currently evaluating the heading
    * structure of the table.
    */
   boolean isEvaluatingHead()
   {
      return this.isEvaluatingHead;
   }
   
   /**
    * @return true or false to indicate if this tag is currently evaluating the extra row
    * structure of the table.
    */
   boolean isEvaluatingExtraRow()
   {
      return this.index == -1 && !this.isEvaluatingHead;
   }
   
   
   /**
    * @return true or false to indicate if this tag is currently evaluating empty content message
    */
   boolean isEvaluatingEmptyContent()
   {
      return this.isEvaluatingEmptyContent;
   }
   
   /**
    * @return
    */
   private String getRowClass()
   {
      return this.index % 2 == 0 ? " class=\"even\"" : " class=\"odd\"";
   }

   /**
    * Retrieves the current item in the iteration for this DataGridTag.
    * @return Returns the current item in the iteration for this DataGridTag.
    */
   public Object getCurrent()
   {
      return current;
   }
   
   int getColumnCount()
   {
      return this.columnCount;
   }
   
   void incrementColumnCount()
   {
      this.columnCount++;
   }
   
   void setrequestContext(RequestContext requestContext)
   {
      this.requestContext = requestContext;
   }

   /**
    * Retrieves the index of the current iteration for this DataGridTag.
    * @return Returns the index of the current iteration for this DataGridTag.
    */
   int getIndex()
   {
      return index;
   }

   /**
    * Retrieves the path for this DataGridTag.
    * @return Returns the path for this DataGridTag.
    */
   public String getPath()
   {
      return path;
   }

   /**
    * Sets the path for this DataGridTag.
    * @param path Sets the path for this DataGridTag.
    */
   public void setPath(String path)
   {
      this.path = path;
   }

   /**
    * Sets the emptyMessageKey for this DataGridTag.
    * @param emptyMessageKey Sets the emptyMessageKey for this DataGridTag.
    */
   public void setEmptyMessageKey(String emptyMessageKey)
   {
      this.emptyMessageKey = emptyMessageKey;
   }
   
   public String getCorrectedPath(String pathToCorrect)
   {
      String commandName = pageContext.getRequest().getAttribute(WebConstants.COMMAND_NAME).toString();
      if(!pathToCorrect.startsWith(commandName))
         throw new BeanInitializationException("Datagrid collection must begin with the command.");
      return pathToCorrect.replaceAll(commandName + "(\\[-?\\d+\\])?\\.", "");
   }

}
