/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import org.springframework.util.StringUtils;

import net.sf.springlayout.util.DateFormatUtil;


/**
 * Generates a date field bound to the specified path and field definition
 * 
 * @author Rob Monie, Adam Boas
 * 
 */
public class DateFieldTag extends InputFieldTag
{
   private static final long serialVersionUID = 7966433596363629518L;
   private boolean validate = true;
   private Calendar minimumDate;
   private Calendar maximumDate;
   private Calendar defaultDate;
   private SimpleDateFormat dateFormatter;
   
   public DateFieldTag()
   {
      super();
   }
   
   
   /**
    * Sets the defaultDate for this DateFieldTag.
    * @param defaultDate Sets the defaultDate for this DateFieldTag.
    */
   public void setDefaultDate(Calendar defaultDate)
   {
      this.defaultDate = defaultDate;
   }


   /**
    * Sets the minimum  date that this DateField instance will allow
    * @param minimumDate that this DateField instance will allow
    */
   public void setMinimumDate(Calendar minimumDate)
   {
      this.minimumDate = minimumDate;
   }
   
   
   /**
    * Sets the maximum  date that this DateField instance will allow
    * @param maximumDate that this DateField instance will allow
    */
   public void setMaximumDate(Calendar maximumDate)
   {
      this.maximumDate = maximumDate;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.InputFieldTag#buildEditModeOutput()
    */
   protected void appendEditModeOutput(StringBuffer output)
   {
      Locale locale = this.getRequestContext().getLocale();
      dateFormatter = DateFormatUtil.getLocalizedDateFormat(locale);
      if (validate)
         prependMap.put(ONBLUR_ATTRIBUTE, this.getDateSpecificValidationRule());
      super.appendEditModeOutput(output);
      
   }
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.InputFieldTag#appendAfterFieldIcons(java.lang.StringBuffer, java.lang.String)
    */
   protected void appendAfterFieldIcons(StringBuffer output, String confirmedPath)
   {
      this.appendDatePicker(output, confirmedPath);
      output.append("<script type=\"text/javascript\">");
      if(!this.renderLiveDatePicker())
      {
         output.append("setDisabled('");
         output.append(confirmedPath);
         output.append("',true);");
      }
      output.append("</script>");
      super.appendAfterFieldIcons(output, confirmedPath);
   }


   /**
    * Returns a string containing HTML for rendering a date picker.
    * 
    * @param dateFormatter for converting date into string
    * @return a string containing HTML for rendering a date picker
    */
   private void appendDatePicker(StringBuffer output, String confirmedPath)
   {
      
      String path = ((HttpServletRequest) pageContext.getRequest()).getContextPath();
      String defaultDateField = "";
      if(this.defaultDate != null)
      {
         defaultDateField = dateFormatter.format(this.defaultDate.getTime()) ;
      }
      
      output.append("<img src=\"");
      output.append(path);
      output.append("/layoutResources/images/icon_calendar.gif");
      output.append("\" alt=\"date picker\" name=\"calendar_");
      output.append(confirmedPath);
      output.append("\" id=\"calendar_");
      output.append(confirmedPath);
      output.append("\" align=\"top\" onclick=\"showCalendarForIcon(this);\" onmouseover=\"over(this, 'calIcon');\" onmouseout=\"out(this,'calIcon');\"/>");

   }
   
   /**
    * Returns <code>true</code> if a live (interactive) date picker
    * should be rendered, or <code>false</code> of a locked one should
    * be rendered instead.
    * 
    * @return <code>true</code> if a live (interactive) date picker
    *         should be rendered
    */
   private boolean renderLiveDatePicker()
   {
      return this.status != null;
   }

   private String getDateSpecificValidationRule()
   {
      StringBuffer output = new StringBuffer();
      output.append("validateDateRangeAndFocus(this,'");
      output.append(this.dateFormatter.toLocalizedPattern());
      output.append("'");
      
      if(this.minimumDate != null && this.maximumDate != null)
      {
         output.append(",'");
         output.append(dateFormatter.format(this.minimumDate.getTime()));
         output.append("','");
         output.append(dateFormatter.format(this.maximumDate.getTime()));
         output.append("')");
      }
      else if(this.minimumDate != null)
      {
         output.append(",'");
         output.append(dateFormatter.format(this.minimumDate.getTime()));
         output.append("')");
      }
      else if(this.maximumDate != null)
      {
         output.append(",null,'");
         output.append(dateFormatter.format(this.maximumDate.getTime()));
         output.append("')");
      }
      else
         output.append(")");
      
      return output.toString();
   }

   /**
    *Sets the validate for this DateFieldTag.
    * @param validate Sets the validate for this DateFieldTag.
    */
   public void setValidate(boolean validate)
   {
      this.validate = validate;
   }
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractFieldTag#getBaseCssClass()
    */
   protected String getBaseCssClass()
   {
      return "dateField";
   }

}
