/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;

import net.sf.springlayout.web.layout.Form;
import net.sf.springlayout.web.layout.PanelForm;
import net.sf.springlayout.web.layout.PanelFormGroup;
import net.sf.springlayout.web.layout.taglib.renderer.PanelFooterRenderer;
import net.sf.springlayout.web.layout.taglib.renderer.PanelTabRenderer;
import net.sf.springlayout.web.util.WebConstants;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.validation.Errors;


/**
 * <p>
 * MultiPanelFormTag outputs the tabs for navigation between panels. It also
 * contains the logic required for displaying tabs with appropriate CSS
 * depending on whether they are the current tab or contain any errors.
 * </p>
 * <p>
 * This tag extends the base Spring tag classhierarcy providing features such as
 * <ul>
 * <li>Access to the <code>WebApplicationContext</code> through the
 * <code>RequestContext</code></li>
 * <li>Escaping of html to be xhtml compliant</li>
 * <li>Locale & messages</li>
 * </ul>
 * </p>
 * 
 * @author Rob Monie
 * 
 */
public class MultiPanelFormTag extends AbstractFormTag
{
   Log log = LogFactory.getLog(MultiPanelFormTag.class);
   private static final long serialVersionUID = -1440738102628543789L;

   private String panelFormGroupBeanName;
   private PanelFormGroup panelFormGroup;
   private String currentPanelFormBeanName;
   private String commandName;
   private PanelForm currentPanel;
   private String panelTabRendererBeanName;
   private PanelTabRenderer panelTabRenderer;
   private String panelFooterRendererBeanName;
   private PanelFooterRenderer panelFooterRenderer;
   private Errors errors;

   /**
    * Used to set the name of the panel group to use for this instance. Should
    * correspond to a valid panelFormGroup bean in the application context
    * 
    * @param panelFormGroupBeanName
    */
   public void setPanelFormGroupBeanName(String panelFormGroupBeanName)
   {
      this.panelFormGroupBeanName = panelFormGroupBeanName;
   }

   /**
    * Sets the name of the current LayoutPanelFormBean to display
    * 
    * @param currentPanelFormBeanName
    */
   public void setCurrentPanelBeanName(String currentPanelBeanName)
   {
      this.currentPanelFormBeanName = currentPanelBeanName;
   }

   /**
    * Sets the name of the panelTabRendererBean to use to render tabs for this
    * panelFormGroup
    * 
    * @param panelTabRendererBeanName
    */
   public void setPanelTabRendererBeanName(String panelTabRendererBeanName)
   {
      this.panelTabRendererBeanName = panelTabRendererBeanName;
   }

   /**
    * Sets the name of the panelFooterRendererBean to use to render the footer
    * for this panelFormGroup
    * 
    * @param panelFooterRendererBeanName
    */
   public void setPanelFooterRendererBeanName(String panelFooterRendererBeanName)
   {
      this.panelFooterRendererBeanName = panelFooterRendererBeanName;
   }

   /**
    * Calls through to panelTabRenderer which renders the panelgroup tabs
    * appropriately for the current context. Renders the start of the panel
    * layout through the panelRenderer
    * 
    * @see org.springframework.web.servlet.tags.RequestContextAwareTag#doStartTagInternal()
    */
   protected int doStartTagInternal() throws JspTagException
   {

      int result = super.doStartTagInternal();
      try
      {
         JspWriter out = pageContext.getOut();
         if(panelTabRenderer != null)
         {
          panelTabRenderer.renderTabs(request, 
                                      panelFormGroup, 
                                      currentPanel, 
                                      errors, 
                                      currentPanel.getFormName(),
                                      out);
         }
         else
         {
            log.debug("No panel tab renderer configured.  Panel Tabs not being rendered.");
         }
      
      }
      catch(Exception e)
      {
         throw new JspTagException(e.getMessage());
      }

      return result;

   }

   /**
    * Closes off layout for the current panel through the panelRenderer
    * 
    * @see javax.servlet.jsp.tagext.Tag#doEndTag()
    */
   public int doEndTag() throws JspException
   {

      try
      {
         // render the tabs for the panelFormGroup
         if (panelFooterRenderer != null)
         {
            panelFooterRenderer.renderFooter(request, panelFormGroup, currentPanel, errors,  this.pageContext.getOut());
         }
         else
         {
            log.debug("No panel footer renderer configured.  Panel footer not being rendered.");
         }

      }

      catch (Exception e)
      {
        throw new JspTagException("Error rendering MultiPanelFormTag");
      }
      return super.doEndTag();
   }


   /*
    * (non-Javadoc)
    * 
    * @see javax.servlet.jsp.tagext.TryCatchFinally#doFinally()
    */
   public void doFinally()
   {
      super.doFinally();
      this.panelFormGroupBeanName = null;
      this.panelFormGroup = null;
      this.currentPanelFormBeanName = null;
      this.currentPanel = null;
      this.panelTabRenderer = null;
      this.errors = null;
   }

   /**
    * Initialises tag and verifiies it's state before proceeding
    */
   protected void init()
   {
      super.init();
      this.panelFormGroup = (PanelFormGroup) ctx.getBean(panelFormGroupBeanName);
      this.currentPanel = (PanelForm) ctx.getBean(currentPanelFormBeanName);

      // no good if no current panel
      if (currentPanel == null)
      {
         throw new NullPointerException("Invalid panel name supplied as current panel");
      }

      if (panelTabRendererBeanName != null && !"".equals(panelFooterRendererBeanName))
      {
         this.panelTabRenderer = (PanelTabRenderer) ctx.getBean(panelTabRendererBeanName);
      }
      if (panelFooterRendererBeanName != null && !"".equals(panelFooterRendererBeanName))
      {
         this.panelFooterRenderer = (PanelFooterRenderer) ctx.getBean(panelFooterRendererBeanName);
      }
      Object commandNameObj = request.getAttribute(WebConstants.COMMAND_NAME);

      if (commandNameObj != null)
      {
         this.commandName = commandNameObj.toString();
         this.errors = this.getRequestContext().getErrors(commandName);
      }
      else
      {
         throw new NullPointerException(
               "layout:panel tag must be used in the context of a spring controller and expects a request attribute "
                     + WebConstants.COMMAND_NAME + " to contain the current controller command name");
      }

   }

   protected boolean isValidationMode(HttpServletRequest request)
   {
      Map stateMap = (Map) request.getAttribute(WebConstants.STATE_MAP);
      if (stateMap != null && "1".equals(stateMap.get(WebConstants.VALIDATE_MODE_PARAM)))
      {
         return true;
      }
      else
      {
         return false;
      }
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.layout.taglib.AbstractFormTag#getCurrentForm()
    */
   protected Form getCurrentForm()
   {
      return this.currentPanel;
   }

}
