/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.commons.lang.ArrayUtils;
import org.springframework.context.NoSuchMessageException;
import org.springframework.validation.ObjectError;
import org.springframework.web.servlet.support.BindStatus;
import org.springframework.web.servlet.tags.HtmlEscapingAwareTag;
import org.springframework.web.servlet.tags.NestedPathTag;
import org.springframework.web.util.ExpressionEvaluationUtils;

/**
 * Shows errors for the path specified... if multiple errors exist for the path,
 * generates an unordered list. Rolls up errors of type "errors.required" into a
 * single error message that can be used to advise the user that one or more
 * mandatory fields are not completed by using the localised message associated
 * with the key <code>errors.requiredFieldErrorsPresent</code>. 
 * 
 * @author Rob Monie
 * 
 */
public class PageErrorTag extends HtmlEscapingAwareTag
{

   private static final long serialVersionUID = 8041505133654275578L;

   private String path;
   private StringBuffer output;
   
   
   /**
    * Primarily for testing this allows the StringBuffer that this tag and it's children use to
    * append their html output to be set and re-acquired.
    *
    * Retrieves the output StringBuffer which this AbstractFieldTag is using to append html.
    * @return Returns the output which this AbstractFieldTag is using to append html.
    */
   public StringBuffer getOutput()
   {
      return output;
   }

   /**
    * Sets the output which this AbstractFieldTag will use to append html.
    * @param output Sets the output which this AbstractFieldTag will use to append html
    */
   public void setOutput(StringBuffer output)
   {
      this.output = output;
   }

   /**
    * Set the path that this tag should apply. 
    */
   public void setPath(String path)
   {
      this.path = path;
   }

   protected final int doStartTagInternal() throws ServletException, JspException
   {
      this.output = new StringBuffer();
      
      String resolvedPath = ExpressionEvaluationUtils.evaluateString("path", this.path, pageContext);

      String nestedPath = (String) pageContext.getAttribute(NestedPathTag.NESTED_PATH_VARIABLE_NAME,
            PageContext.REQUEST_SCOPE);
      if (nestedPath != null)
      {
         resolvedPath = nestedPath + resolvedPath;
      }
      BindStatus status = null;
      try
      {
         status = new BindStatus(getRequestContext(), resolvedPath, isHtmlEscape());
      }
      catch (IllegalStateException ex)
      {
         throw new JspTagException(ex.getMessage());
      }

      List errors = this.getErrorMessages(status);
      
      if (errors.size() > 0)
      {
         output.append("<ul id=\"messages\" class=\"errors\">\n");
         for (Iterator it = errors.iterator(); it.hasNext();)
         {
            output.append("<li>" + it.next().toString() + "</li>\n");
         }
         output.append("</ul>\n");
      }
      
      try
      {
         this.pageContext.getOut().write(output.toString());
      }
      catch (IOException e)
      {
         throw new JspTagException(e.getMessage());
      }

      return TagSupport.SKIP_BODY;
   }

   /**
    * Extract the error messages from the ObjectError list.
    */
   private List getErrorMessages(BindStatus status) throws NoSuchMessageException
   {
      boolean hasRequiredFieldErrors = false;

      List errorMessages = new ArrayList();
      for (int i = 0; i < status.getErrors().getAllErrors().size(); i++)
      {
         ObjectError error = (ObjectError) status.getErrors().getAllErrors().get(i);
         if (ArrayUtils.contains(error.getCodes(), "errors.required"))
         {
            hasRequiredFieldErrors = true;
         }
         else
         {
            errorMessages.add(this.getRequestContext().getMessage(error, this.isHtmlEscape()));
         }
      }
      if (hasRequiredFieldErrors)
      {
         errorMessages.add(this.getRequestContext().getMessage("errors.requiredFieldErrorsPresent"));
      }
      return errorMessages;
   }
}
