/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.layout.taglib.renderer;

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import net.sf.springlayout.web.layout.PanelForm;
import net.sf.springlayout.web.layout.PanelFormGroup;
import net.sf.springlayout.web.security.PageAccessManager;
import net.sf.springlayout.web.util.RequestUtilSupport;
import net.sf.springlayout.web.util.WebConstants;

import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.util.HtmlUtils;


/**
 * Default implementation of the PanelTabRenderer
 * 
 * @author monier
 * 
 */
public class DefaultPanelTabRenderer implements PanelTabRenderer
{


   /**
    * Renders tabs, providing appropriate css to highlight
    * <ul>
    * <li>The current tab / panel</li>
    * <li>Panels with errors</li>
    * <li>Disabled Panels</li>
    * </ul>
    * 
    * @param request
    * @param panelFormGroup
    * @param currentPanelName
    * @param errors
    * @param writer
    *           the jsp writer
    */
   public void renderTabs(HttpServletRequest request, PanelFormGroup panelFormGroup, PanelForm currentPanelForm, Errors errors,
         String formName, Writer writer) throws IOException, ServletRequestBindingException
   {
      StringBuffer msg = new StringBuffer();
      String globalErrorDisplayElementId = "tabBarSubBar";
      msg.append("<ul class=\"panelTabs\">");
      String tabBarSubBar = "";
      Map stateMapForUrl = RequestUtilSupport.getStateMapForUrl((Map)request.getAttribute(WebConstants.STATE_MAP));
            
      for (Iterator it = panelFormGroup.getPanelForms().iterator(); it.hasNext();)
      {
         PanelForm panelForm = (PanelForm) it.next();
         String tabHtml;
         if (panelForm.equals(currentPanelForm))
         {
            if (panelForm.hasErrors(errors))
            {
               tabHtml = "<li class=\"tab currentError\">" + HtmlUtils.htmlEscape(panelForm.getPanelLabel()) + "</li>";
               tabBarSubBar = "<div id=\"" + globalErrorDisplayElementId + "\" class=\"error\"></div>";
            }
            else
            {
               tabHtml = "<li class=\"tab current\">" + HtmlUtils.htmlEscape(panelForm.getPanelLabel()) + "</li>";
               tabBarSubBar = "<div id=\"" + globalErrorDisplayElementId + "\"></div>";
            }
         }
         else
         {
            if (panelForm.isDisabled(request))
            {
               tabHtml = "<li class=\"tab disabled\">" + HtmlUtils.htmlEscape(panelForm.getPanelLabel()) + "</li>";
            }
            else
            {
               String errorClass = panelForm.hasErrors(errors) ? "class=\"tab error\"" : "class=\"tab\"";

               if (PageAccessManager.getPageAccessHolder().isEditMode())
               {
                  tabHtml = "<li "  + errorClass + "><a href=\"#\" onclick=\"changePanel('" + formName + "'," + panelForm.getIndex()
                        + ");\" title=\"" + HtmlUtils.htmlEscape(panelForm.getPanelLabel()) + "\">"
                        + panelForm.getPanelLabel() + "</a></li>";

               }
               else
               {
                  tabHtml =  "<li "  + errorClass + "><a href=\"" + panelForm.getFormAction() + "?" +  this.getQueryString(stateMapForUrl);
                  tabHtml += "\" title=\"" + HtmlUtils.htmlEscape(panelForm.getPanelLabel()) + 
                     "\">" + panelForm.getPanelLabel() + "</a></li>";
               }
            }
         }
         msg.append(tabHtml );
      }

      msg.append("</ul>");
      
      msg.append(tabBarSubBar);
      msg.append("<script language=\"JavaScript\" type=\"text/javascript\">");
      msg.append("var globalErrorDisplayElement = $('");
      msg.append(globalErrorDisplayElementId);
      msg.append("');");
      msg.append("</script>");
      writer.write(msg.toString());

   }
   
   /**
    * Helper method to build a properly formatted query string for the current
    * state of the system.
    * 
    * @param params
    * @param linkIndex
    * @return properly formatted query string.
    */
   private String getQueryString(Map params)
   {
      StringBuffer buffy = new StringBuffer(); 
      if(params != null)
      {
         Set keys = params.keySet();
         for(Iterator it = keys.iterator(); it.hasNext(); )
         {
            String key = (String)it.next();
            buffy.append(key + "=" + params.get(key));
            if(it.hasNext())
               buffy.append("&");
         }
      }
      return buffy.toString();
   }
}
