/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.security;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

public abstract class AbstractPageAccessResolver implements PageAccessResolver
{

   private boolean runOnFormSubmission;
  
   
   /**
    * Set whether the pageAccessResolver should run on formSubmission.  Default is false.
    * @param runOnFormSubmission the runOnFormSubmission to set
    */
   public void setRunOnFormSubmission(boolean runOnFormSubmission)
   {
      this.runOnFormSubmission = runOnFormSubmission;
   }
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.security.PageAccessResolver#isRunOnFormSubmission()
    */
   public boolean isRunOnFormSubmission()
   {
      return this.runOnFormSubmission;
   }

   /**
    * Firstly determines the maximum access allowed for the current user in the
    * current context, by calling <code>getPageAccessLevel</code>,
    * downgrading the access where necessary. Secondly based on the outcome of
    * this, initialises a page function map by calling
    * <code>getPageFunctionPermissions</code> and stores functions against
    * PageFunctionStates in the request for later reference via the
    * <code>getPageFunctionState</code> method.
    * 
    * @param request
    *           the current request
    * @param object
    *           the current command object
    * @see net.sf.springlayout.web.security.PageAccessResolver#initialisePageFunctions(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object)
    */
   public void initialisePageAccess(HttpServletRequest request, Object object)
   {
      
      PageAccessHolder pageAccessHolder = PageAccessManager.getPageAccessHolder();

      if (!isFormSubmission(request))
      {
         AccessLevel maxAllowedAccessLevel = getPageAccessLevel(request, object);

         if (pageAccessHolder.isEditMode() && maxAllowedAccessLevel.equals(AccessLevel.READ_ACCESS))
         {
            pageAccessHolder.setAccessLevel(AccessLevel.READ_ACCESS);
         }
         else if (maxAllowedAccessLevel.equals(AccessLevel.NO_ACCESS))
         {
            throw new AccessDeniedException("Insufficient access to " + object.getClass().getName() + " with id: "
                  + object.toString());
         }
      }

      PageFunctionStateMap pageFunctions = getPageFunctionPermissions(request, object, pageAccessHolder.isEditMode());
      
      pageAccessHolder.setPageFunctions(pageFunctions);

      for(Iterator it = this.defineReadOnlyFields(request, object).iterator(); it.hasNext();)
      {
         String fieldKey = it.next().toString();
         pageAccessHolder.addReadOnlyField(fieldKey);
      }
   }

   /**
    * Implement this method to return a map of <code>PageFunctionState</code>s
    * for the current page with the following structure: String
    * functionName:PageFunctionState.
    * 
    * @param request
    *           the current request
    * @param object
    *           the current command object
    * @param editMode
    *           are we opening in edit mode
    * @return map of function states
    */
   protected abstract PageFunctionStateMap getPageFunctionPermissions(HttpServletRequest request, Object object, boolean editMode);

   /**
    * Looks up the pageFunction map from the request and queries it for the
    * function name requested returning a page function state.
    * 
    * @param fucntionName
    *           the name of the function to get a state for
    * @see net.sf.springlayout.web.security.PageAccessResolver#getPageFunctionState(javax.servlet.http.HttpServletRequest,
    *      java.lang.String)
    */
   public PageFunctionState getPageFunctionState(String functionName)
   {
      return PageAccessManager.getPageAccessHolder().getPageFunctionState(functionName);
   }

   /**
    * set the page function state
    * 
    * @param functionName
    * @param pageFunctionState
    */
   public void setPageFunctionState(String functionName, PageFunctionState pageFunctionState)
   {
      PageAccessManager.getPageAccessHolder().setPageFunctionState(functionName, pageFunctionState);
   }

   /**
    * Default implementation that defines no fields as readonly. Override to
    * return a list of Strings for field paths to disable.
    * 
    * @param request
    * @param object
    * @return
    */
   protected List defineReadOnlyFields(HttpServletRequest request, Object object)
   {
      List list = new ArrayList();
      return list;
   }

   /**
    * Implement method to get the page access level for the current page
    * 
    * @param the
    *           current request
    * @param object
    *           the current command object
    * @return
    */
   protected abstract AccessLevel getPageAccessLevel(HttpServletRequest request, Object object);

   /**
    * Tests if a post or a get
    * 
    * @param request
    * @return
    */
   protected boolean isFormSubmission(HttpServletRequest request)
   {
      return "POST".equals(request.getMethod());
   }
}
