/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.security;

import java.util.HashMap;
import java.util.Map;

import net.sf.springlayout.web.layout.LayoutUtils;


/**
 * Maintains the page read /edit state for a request and potential for determining readonly fields 
 * and page function states at run time. This class is used extensively by the layout field tags to 
 * determine how to display a field on a page. PageAccessHolder should be accessed via the 
 * <code>PageAccessManager</code>
 * 
 * @see net.sf.springlayout.web.security.PageAccessManager
 * @author rob monie
 * 
 */
public class PageAccessHolder
{

   
   private AccessLevel accessLevel;

   /**
    * Holds any fields to be treated as read / display only for this request
    */
   private Map readOnlyFields;
   
   /**
    * Map of page functions and their associated <code>PageFunctionState</code>
    */
   private PageFunctionStateMap pageFunctions;

   /**
    * Default constructor. Sets editMode to false by default and initialises the readOnlyFields map.
    */
   public PageAccessHolder()
   {
      this.accessLevel = AccessLevel.NO_ACCESS;
      this.readOnlyFields = new HashMap();
      this.pageFunctions = new PageFunctionStateMap();
   }
   
   /**
    * Gets the access level
    * @return
    */
   public AccessLevel getAccessLevel()
   {
      return accessLevel;
   }


   /**
    * Sets the access level
    * @param accessLevel
    */
   public void setAccessLevel(AccessLevel accessLevel)
   {
      this.accessLevel = accessLevel;
   }




   /**
    * Gets the edit mode for the page or request
    * 
    * @return
    */
   public boolean isEditMode()
   {
      return this.accessLevel.equals(AccessLevel.EDIT_ACCESS);
   }

   /**
    * Sets the pageAccessHolder to edit mode
    * 
    * @param editMode
    */
   public void setEditMode()
   {
      this.accessLevel = AccessLevel.EDIT_ACCESS;
   }
   
   /**
    * Gets the read mode for the page or request
    * 
    * @return
    */
   public boolean isReadMode()
   {
      return this.accessLevel.equals(AccessLevel.READ_ACCESS);
   }
   
   /**
    * Sets the pageAccessHolder to read mode
    * 
    * @param editMode
    */
   public void setReadMode()
   {
      this.accessLevel = AccessLevel.READ_ACCESS;
   }
   
   /**
    * gets the noAccess Property. 
    * @return
    */
   public boolean isNoAccess()
   {
      return this.accessLevel.equals(AccessLevel.NO_ACCESS);
   }

   /**
    * sets the pageAccessHolder to no access
    * @param noAccess
    */
   public void setNoAccess()
   {
      this.accessLevel = AccessLevel.NO_ACCESS;
   }
   
   /**
    * Sets the read only field map for the page.  
    * Fields found in this map will be treated as readonly if the page is in edit mode.
    * @param readOnlyFields
    */
   public void setReadOnlyFields(Map readOnlyFields)
   {
      this.readOnlyFields = readOnlyFields;
   }

   /**
    * Add a readonly field to the map by field key
    * 
    * @param fieldKey
    */
   public void addReadOnlyField(String fieldKey)
   {
      readOnlyFields.put(fieldKey, Boolean.TRUE);
   }

   /**
    * Remove a readonly field from the map
    * 
    * @param fieldKey
    */
   public void removeReadOnlyField(String fieldKey)
   {
      readOnlyFields.remove(fieldKey);
   }

   /**
    * <p>
    * Determines whether a field is read only by queryiing the readonlyField
    * map. At this stage indexed field paths will be stripped of indexes to
    * perform a match so any indexed paths such as
    * <code>incident.vessels[2].vesselName</code> would match against a path
    * of <code>index.vessels.vesselName</code> in the readonlyFields map.
    * For this reason all fields in the readonlyFields map must also be stripped
    * of specific indexes. Future releases could support specific indexes if
    * required.
    * </p>
    * <p>
    * If the page is not in edit mode then returns true for every field without
    * checking.
    * </p>
    * 
    * @param path the field path
    * @return true if the field is readonly
    */
   public boolean isFieldReadOnly(String path)
   {
      if (this.accessLevel.equals(AccessLevel.EDIT_ACCESS))
      {
         String strippedFieldKey = LayoutUtils.getStrippedPropertyPath(path);

         if (readOnlyFields.get(strippedFieldKey) == null)
         {
            return false;
         }
         else
         {
            return true;
         }
      }
      else
      {
         return true;
      }
   }
   
   /**
    * Sets the pageFunctionState in the pageAccessHolder
    * 
    * @param functionName
    *           name of the function to set
    * @param pageFunctionState
    *           state of the function
    */
   public  void setPageFunctionState(String functionName, PageFunctionState pageFunctionState)
   {

         this.pageFunctions.put(functionName, pageFunctionState);

   }
   
   /**
    * Looks up the pageFunction map from the threadLocal variable and queries it
    * for the function name requested returning a page function state.
    * 
    *  
    * @see net.sf.springlayout.web.security.PageAccessResolver#getPageFunctionState(javax.servlet.http.HttpServletRequest,
    *      java.lang.String)
    */
   public PageFunctionState getPageFunctionState(String functionName)
   {

      if (this.pageFunctions.get(functionName) == null)
      {
         return PageFunctionState.HIDDEN;
      }
      else
      {
         return (PageFunctionState) this.pageFunctions.get(functionName);
      }
     

   }

   /**
    * Gets the pageFunctionMap
    * @return
    */
   public PageFunctionStateMap getPageFunctions()
   {
      return pageFunctions;
   }

   /**
    * Sets the pageFunctionMap
    * @param pageFunctions
    */
   public void setPageFunctions(PageFunctionStateMap pageFunctions)
   {
      this.pageFunctions = pageFunctions;
   }
   
   
}
