/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.validator;

import javax.servlet.http.HttpServletRequest;

import net.sf.springlayout.web.layout.OnPageFieldInstance;
import net.sf.springlayout.web.validator.condition.Condition;

import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.NullValueInNestedPathException;
import org.springframework.validation.Errors;

/**
 * Abstract validation rule providing common functionality for concrete <code>ValidationRule</code>
 * implementations
 * 
 * @author rob monie
 * 
 */
public abstract class AbstractValidationRule implements ValidationRule
{

   private Condition condition;

   /**
    * @return the condition
    */
   public Condition getCondition()
   {
      return condition;
   }

   /**
    * Set a condition to evaluate to determine whether to run this validation rule
    * 
    * @param condition the condition to set
    */
   public void setCondition(Condition condition)
   {
      this.condition = condition;
   }

   /**
    * Gets a fields value based on its fieldKey / path. Handles NullValueInNestedPathException
    * 
    * @param obj
    * @param fieldKey
    * @return
    */
   public Object getFieldValue(Object obj, String fieldKey)
   {

      BeanWrapper beanWrapper = new BeanWrapperImpl(obj);

      Object fieldValue = null;

      try
      {
         fieldValue = beanWrapper.getPropertyValue(fieldKey);
      }
      catch (NullValueInNestedPathException e)
      {
         // do nothing - fieldvalue remains null
      }

      return fieldValue;
   }

   /**
    * Calls upon condition to determine whether the validation rule should applied or not then
    * conditionally validates.
    * 
    * @param obj the command object on which to validate
    * @param fieldKey the fieldKey or path to the field in the object
    * @return
    */
   public void conditionallyValidate(HttpServletRequest request, Object obj, Errors errors,
            String fieldKey)
   {
      BeanWrapper beanWrapper = new BeanWrapperImpl(obj);

      if ((condition == null) || (condition != null && condition.evaluate(fieldKey, beanWrapper)))
      {
         this.validate(request, obj, errors, fieldKey);
      }

   }

   /**
    * Validate the object
    * 
    * @param request
    * @param obj
    * @param errors
    * @param fieldKey
    */
   protected abstract void validate(HttpServletRequest request, Object obj, Errors errors,
            String fieldKey);

   /*
    * (non-Javadoc)
    * 
    * @see net.sf.springlayout.web.validator.ValidationRule#getConditionalJavascript()
    */
   public String getConditionalJavascript(OnPageFieldInstance fieldInstance, Object command,
            String formName)
   {

      StringBuffer script = new StringBuffer();

      if (condition != null)
      {
         script.append(condition.getJavascript(fieldInstance, command, formName));
         script.append(" ? ");
         script.append(getJavascript());
         script.append(" : true;");
      }
      else
      {
         script.append(getJavascript());
         script.append(";\n");
      }

      return script.toString();
   }

   /**
    * Get the javascript used to validate this rule
    * 
    * @return
    */
   public abstract String getJavascript();

}
