/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.validator;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.validation.Errors;

/**
 * Provides number  validation against the supplied range values.
 * 
 * @author Rob Monie, Adam Boas
 *
 */
public class IntegerNumberRangeValidator extends AbstractValidationRule
{
   private static Log log = LogFactory.getLog(IntegerNumberRangeValidator.class);

   private int minValue;
   private int maxValue;

   public IntegerNumberRangeValidator() { }

   /**
    * Construct Validator with min and max values
    * @param minValue
    * @param maxValue
    * @throws NullPointerException
    */
   public IntegerNumberRangeValidator(int minValue, int maxValue)
   {
      this.minValue = minValue;
      this.maxValue = maxValue;
   }
   

   /**
    * Sets the maxValue for this IntegerNumberRangeValidator.
    * @param maxValue Sets the maxValue for this IntegerNumberRangeValidator.
    */
   public void setMaxValue(int maxValue)
   {
      this.maxValue = maxValue;
   }

   /**
    * Sets the minValue for this IntegerNumberRangeValidator.
    * @param minValue Sets the minValue for this IntegerNumberRangeValidator.
    */
   public void setMinValue(int minValue)
   {
      this.minValue = minValue;
   }

   /**
    * Validates object against min and max values.
    * 
    * @see net.sf.springlayout.web.validator.ValidationRule#validate(javax.servlet.http.HttpServletRequest,
    *      java.lang.Object, org.springframework.validation.Errors,
    *      java.lang.String)
    */
   public void validate(HttpServletRequest request, Object obj, Errors errors,
         String fieldKey)
   {

      Object fieldValue = this.getFieldValue(obj, fieldKey);

      if (fieldValue != null)
      {
         try
         {

            int numValue = Integer.parseInt(fieldValue.toString());

            if (numValue < minValue || numValue > maxValue)
            {

               log.debug("Failed number range validation. Value " + numValue + " is not between " + 
                     minValue + " and " + maxValue);
              
               // need to remove the decimal place if it ends in ".0"
               String strMinValue = "" + minValue;
               String strMaxValue = "" + maxValue;

               Object[] args = { strMinValue, strMaxValue };
               errors.rejectValue(fieldKey, "errors.numberRangeValidation",
                     args, "errors.numberRangeValidation");
            }

         }
         catch (NumberFormatException e)
         {
            log.debug("Attempt to validate a non-numeric value");
            errors.rejectValue(fieldKey, "typeMismatch.java.lang.Double");
         }

      }
   }

   public boolean isValid(Object obj, String fieldKey)
   {
      Object fieldValue = this.getFieldValue(obj, fieldKey);

      if (fieldValue != null)
      {
         try
         {
            int numValue = Integer.parseInt(fieldValue.toString());
            if (numValue < minValue || numValue > maxValue)
            {
               return false;
            }
         }
         catch (NumberFormatException e)
         {
            return false;
         }
      }
      return true;
   }


   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.ValidationRule#getJavascript()
    */
   public String getJavascript()
   {
      StringBuffer script = new StringBuffer();
      script.append("validateRange(field,");
      script.append(this.minValue);
      script.append(",");
      script.append(this.maxValue);
      script.append(",alerts)");
      return script.toString();
   }


// /**
// * Ensures that bean is configured correctly in spring at start up.
//    * 
//    * @throws Exception
//    */
//   public void afterPropertiesSet() throws Exception
//   {
//      if (minValue == null || maxValue == null)
//      {
//         throw new ValidationConfigurationException(
//               "Invalid range specified.  Ensure both minValue and maxvalue attributes have been set.");
//      }
//
//   }

}
