/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.validator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.InvalidPropertyException;
import org.springframework.beans.NotReadablePropertyException;
import org.springframework.beans.NullValueInNestedPathException;

/**
 * @author Adam Boas
 * Created on 15/05/2006
 */
public class NestedPathResolver
{
   private static final Log log = LogFactory.getLog(NestedPathResolver.class);
   private BeanWrapper wrapper;
   
   public NestedPathResolver(Object command)
   {
      this.wrapper = new BeanWrapperImpl(command);
   }
   
   public List getValidPaths(String path)
   {
      List paths = new ArrayList();
      this.resolvePathsRecursively(paths, "", path);
      return paths;
   }
   
   private void resolvePathsRecursively(List paths, String pathBase, String pathRemainder)
   {
      try
      {
         String path = mergeBaseAndRemainder(pathBase, pathRemainder);
         this.wrapper.getPropertyValue(path);
         paths.add(path);
      }
      catch(NullValueInNestedPathException nve)
      {
         //end of path reached without matching the required path.
      }
      catch(NotReadablePropertyException e)
      {
         log.debug("Possible collection detected in nested path, attempting to resolve...");
         String newBase;
         try
         {
            int strIndex = pathRemainder.indexOf('.');
            if (strIndex == -1) {
               String errpath = mergeBaseAndRemainder(pathBase, pathRemainder);
               throw new InvalidPropertyException(wrapper.getWrappedClass(), errpath + " at " + pathBase, "Invalid path in layout form definition", e);
            }
               
            newBase= mergeBaseAndRemainder(pathBase, 
               pathRemainder.substring(0, strIndex));
         }
         catch(StringIndexOutOfBoundsException siobe)
         {
            //this will happen is there is an invalid path in a form definition.
            String errpath = mergeBaseAndRemainder(pathBase, pathRemainder);
            throw new InvalidPropertyException(wrapper.getWrappedClass(), errpath + " at " + pathBase, "Invalid path in layout form definition", siobe);
         }
         String newRemainder = pathRemainder.substring(pathRemainder.indexOf('.') + 1);
         Class newBaseType = this.wrapper.getPropertyType(newBase);
         if (newBaseType == null)
         {
            String errpath = mergeBaseAndRemainder(newBase, newRemainder);
            throw new NotReadablePropertyException(wrapper.getWrappedClass(),errpath + " at " + newBase);
         }
         if(typeIsIndexedCollection(newBaseType))
         {
            Iterator iterator = getIterator(newBase);
            for (int index = 0; iterator.hasNext(); index++)
            {
               resolvePathsRecursively(paths, newBase + "[" + index + "]", newRemainder);
               iterator.next();
            }
         }
         else if (typeIsMap(newBaseType))
         {
            Map map = (Map) this.wrapper.getPropertyValue(newBase);
            for (Iterator iter = map.keySet().iterator(); iter.hasNext();) {
               String key = (String)iter.next();
               resolvePathsRecursively(paths, newBase + "[" + key + "]", newRemainder);
            }
         }
         else
         {
            resolvePathsRecursively(paths, newBase, newRemainder);
         }
      }
   }

   private String mergeBaseAndRemainder(String pathBase, String pathRemainder)
   {
      return (pathBase.length() == 0)? pathRemainder : pathBase + "." + pathRemainder;
   }
   
   private boolean typeIsMap(Class type)
   {
      return (Map.class.isAssignableFrom(type));
   }

   private boolean typeIsIndexedCollection(Class type)
   {
      return (type.isArray() || List.class.isAssignableFrom(type));
   }
   
   private Iterator getIterator(String path)
   {
      Object collection = this.wrapper.getPropertyValue(path);
      if (collection instanceof List)
      {
         return ((List) collection).iterator();
      }
      else
      {
         return Arrays.asList((Object[]) collection).iterator();
      }
   }
}
