/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.springlayout.web.validator.condition;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

import net.sf.springlayout.web.layout.LayoutFormConfigurationException;
import net.sf.springlayout.web.layout.OnPageFieldInstance;
import net.sf.springlayout.web.util.FieldPathUtils;

import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.InvalidPropertyException;
import org.springframework.util.StringUtils;


/**
 * @author Adam Boas 
 * Created on 18/05/2006
 */
public class PathBasedCondition implements Serializable, Condition
{
   private static final long serialVersionUID = -3222010750671830901L;
   private static final Map EVALUATORS = new HashMap();
   static
   {
      EVALUATORS.put("==", new EqualsEvaluator());
      EVALUATORS.put("!=", new NotEqualsEvaluator());
      EVALUATORS.put(">", new GreaterThanEvaluator());
      EVALUATORS.put(">=", new GreaterThanOrEqualToEvaluator());
      EVALUATORS.put("<", new LessThanEvaluator());
      EVALUATORS.put("<=", new LessThanOrEqualToEvaluator());
   }
   private String path;
   /**
    * Represents the evaluation condition for comparison. Valid values are:
    * <ul>
    * <li>'!='</li>
    * <li>'='</li>
    * <li>'>'</li>
    * <li>'>='</li>
    * <li>'<'</li>
    * <li>'<='</li>
    * </ul>
    */
   private String operator;
   private String value;
   private boolean onThisForm = true;

   /** default constructor */
   public PathBasedCondition()
   {
      super();
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#setOperator(java.lang.String)
    */
   public void setOperator(String operator)
   {
      this.operator = this.parseOperator(operator);
   }
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#getOperator()
    */
   public String getOperator()
   {
      return this.operator;
   }

   /**
    * @param path
    */
   public void setPath(String path)
   {
      this.path = path;
   }
   
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#getPath()
    */
   public String getPath()
   {
      return this.path;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#setValue(java.lang.String)
    */
   public void setValue(String value)
   {
      this.value = value;
   }
   
   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#getValue()
    */
   public String getValue()
   {
      return this.value;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#evaluate(org.springframework.beans.BeanWrapper, java.lang.String)
    */
   public boolean evaluate(String fieldId, BeanWrapper wrappedObject)
   {
      // TODO:Think about how we should handle null values. (AB)
      Evaluator evaluator = (Evaluator) EVALUATORS.get(operator);
      Object target = null;
      try
      {
         target = wrappedObject.getPropertyValue(
            FieldPathUtils.correctDependentPath(fieldId,
                                                   this.path));
      }
      catch(InvalidPropertyException ipe)
      {
         //do nothing here, this could well happen on a new record.
      }

      if (target == null)
      {
         return false;
      }
      return evaluator.evaluate(target, this.value);
   }

   
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#getJavascript(java.lang.String)
    */
   public String getJavascript(OnPageFieldInstance instance, Object command, String formName)
   {
      if(!this.onThisForm)
      {
         if(this.evaluate(instance.getFieldId(), new BeanWrapperImpl(command)))
            return "true";
         else
            return "false";
      }
      String actualPath = FieldPathUtils.correctDependentPath(instance.getFieldId(), this.getPath());
      StringBuffer script = new StringBuffer();
      //TODO: Should this be using $ ???
      script.append("$F('");
      script.append(actualPath);
      script.append("') ");
      script.append(this.operator);
      script.append(" ");
      
      //If value is a boolean then convert it to 1/0 for true/false
      if("true".equals(this.value))
         script.append("1 ");
      else if("false".equals(this.value))
         script.append("0 ");
      else
      {
         //try to coerce value to a number
         try
         {
            BigDecimal number = new BigDecimal(this.value);
            script.append(this.value);
            script.append(" ");
         }
         catch(NumberFormatException nfe)
         {
            script.append("'");
            script.append(this.value);
            script.append("' ");
         }
      }

      return script.toString();
   }
      
   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#getAfterJavascriptScript(java.lang.String)
    */
   public String getPostProcessingJavascript(OnPageFieldInstance instance, String formName)
   {
      if(!this.onThisForm)
      {
         return "";
      }
      
      String actualPath = FieldPathUtils.correctDependentPath(instance.getFieldId(), this.getPath());
      StringBuffer script = new StringBuffer();
      
      //we don't know if the dependent field is a grouped field or not (Ie. radiobutton) so get the groups master field by id instead.
      script.append("registerFieldAsMandatoryDependent($('");
      script.append(actualPath);
      script.append("'), document.forms['");
      
      script.append(formName);
      script.append("']['");
      script.append(instance.getFieldId());
      
      //if the field is grouped then we want to register against the first field as the master
      if(instance.isGrouped())
      {
         script.append("'][0]);\n");
      }
      else
      {
         script.append("']);\n");
      }
      return script.toString();
   }

   private String parseOperator(String operator)
   {
      String parsedOperator = operator.replaceAll("&gt;", ">");
      parsedOperator = operator.replaceAll("&lt;", "<");
      return parsedOperator;
   }

   /**
    * Sets the onThisForm for this PathBasedCondition.
    * @param onThisForm Sets the onThisForm for this PathBasedCondition.
    */
   public void setOnThisForm(boolean onThisForm)
   {
      this.onThisForm = onThisForm;
   }

   /**
    *
    * Retrieves the onThisForm for this PathBasedCondition.
    * @return Returns the onThisForm for this PathBasedCondition.
    */
   public boolean isOnThisForm()
   {
      return onThisForm;
   }

   /* (non-Javadoc)
    * @see net.sf.springlayout.web.validator.condition.Condition#validate()
    */
   public void afterPropertiesSet()
   {
      if (! StringUtils.hasText(path))
         throw new LayoutFormConfigurationException( "A PathBasedCondition is not configured correctly.\n");
      Evaluator evaluator = (Evaluator)EVALUATORS.get(this.operator);
      if(evaluator == null)
         throw new LayoutFormConfigurationException( "Unsupported operator in PathBasedCondition: " 
                  + this.path + "\n");
   }
   
}
